;  ORAC5xx.S - a strong checkers playing program in 6502 assembler, for PET/64
;   Copyright A.Millett, 1980-2024. Released as Freeware.
;------------------------------------------------------------------------------
;	Resource of SAGE 120. Copyright A.Millett.
;      Take SAGE120C.ASM, (patched PET version) convert to CA65 compatible asm..
;  version that compiles/runs (under PETSIM..)
;  (NOTE: needs PET type setup with VDU at $8000, Basic start at $0401. Custom chrset at $8800.
;     so run PET.ORACSET.7 first - c64 PETSIM with Orac draughts piece chrset)
;-> SAGE400.S (18.5.12)
;  make purely source-level changes to tidy/improve code readability.
;    (NOTE ERASEPIECE etc: not working in 400..)
;  make version compiles identical binary to SAGE120MC.PRG  (ie  FC SAGE120MC,PRG SAGE401.PRG /B)
;-> SAGE401.S (18.5.12)
;  cont to make code more readable, put back comments from "ORAC 2.15 MAIN" and "ORAC i33 MAIN".
;-> SAGE402.S (20.5.12)
;  correct ref to Dangerous ptrs, and find more static ptr definitions in code, change to proper variable refs
;  Imp KEEP_ORIG_CODE define - force orig binary compile (enables easy verification of changes working)
;  Experiment shifting main code up to $083c start (SYS 4726 starts). After fixing bugs, Works ok..
;-> SAGE403.S (20.5.12)
;  Lable/annotate/tidy more code
;  Make 64 version with code start $083c, adding basic line 10 SYS 4726. Works ok..
;-> SAGE404.S (21.5.12)
;  Imp KEEP_ORIG_CODE to remove unused/redundant code. 
; Many original tricky ORAC bugs found if page boundarys change! 
;   new routine for selecting weight table (selmancent) needed.
;   if (tmovestack) crosses a page, SAGE crashed!
;   testnotes-to verify code working as before: (same for SAGE120MC.PRG or ORAC88 with $1909=#$60 to kill rand)
;     iq1 ,auto:Bwin, (WK15,30:B26,K27) Black Clock 1:34.
;     iq10,auto:Wwin, (WK5,K10,K19,K22:BK13) White clock approx 2:52.
;     iq17,auto:Bwin, (WK17:B1,K18,K27)  Black Clock 4:14
;     iq18,auto:draws,(WK5,K18,28:BK6,K30) 4 move shuffle.
;     iq23,auto:Bwin  (WK1,13:BK5,K10,12,K22,K25,K31) Black clock 26:34.
;     iq25,auto:Wwin  (WK10,K11,K19,30:B21,K28)  White clock 44.14 (s407=40:54)
;   imp BASIC stub at $0801
;   imp SMALLER_CODE, disables S/L, 64 printer and other things to reduce size.
;   compiles 5737 with SMALLER_CODE, to 7967 with KEEP_ORIG_CODE set. (fc SAGE120MC identical)
;-> SAGE405.S (22.5.12)
;   Unroll a number of patches, removing more redundant code/data
;   Use Tape buffer ($33c) and input buffer ($200) to store some vars
;-> SAGE406.S (23.5.12)   (SAGE406.prg = 4960 bytes)
;   Reduce ply stack to 24, movestack to 96
;   Scavange more lowmem for table storage.
;   Put tboard into zero page. (Note $03-$8f is BASIC zp, can be reused.)
;   About 10% faster with zero page variables.
;-> SAGE407.S (24.5.12)   (SAGE407.prg =  3880 bytes)
;   Imp USE_CA65=1 kludge. change .hibyte setiq-1.. to .byte >(setiq-1).. Very minor changes (for ASM502).
;-> SAGE409.S (6.11.16)
; Regress to original ORAC name for 6502 version. Improve comments etc..
;-> ORAC500.S (4.5.24)
;  Remove old KEEP_ORIG_CODE, remove old patches etc.  Test compile to same as before.
;  Start imp IS_PET and IS_C64, separate PET/64 stuff properly.
;-> ORAC501.S (5.5.24)
;  Add mode comments, etc.
;  Imp USE_OLD_CODE
;  Imp SHOW_MOVE_INFO, imp simple node count+display.
;-> ORAC502.S (6.5.24)
;  Imp display best move/eval.
;  Tidy up more..
;-> ORAC503.S (7.5.24)
;  Update version text etc..
;-> ORAC504.S (30.5.24)
;  imp init_graphics - Copy chrset from $d000 to $3000, imp VICmem ($d018), C64memctl, pCHRSET etc
;  Save/load pos not working, Debug/tidy brdsave/load..
;-> ORAC505.S (31.5.24)
;  Tidy/remove more patches. Tidy displaymsg routine, imp tmsgwhat, tmsgmove etc..
;  Remove nops etc.
;  Imp textout, simpler/compact text display. Imp tmsghelp, and front cmd summary help screen.
;-> ORAC506.S (1.6.24)
;  Remove old C64 print routines (patch64printmove)
;  Imp USE_CLOCK. Tidy up clock/irq routines..
;  Imp io_initialise, io_getkey, io_sound, io_wait_retrace,  Remove machine specific stuff from main code..
;  Imp PET version..
;-> ORAC507.S (2.6.24)
;  Bug in PET version. Imp clr zero page storage (zpStart to zpEnd) cures, but needs better debugging. (if zp area init to 1, bug persists)
;  Imp io_wait_jiffy. Imp PET key_buffer_size, zp_jiffy_low, vectirq, romirq. 
;  Odd bugs when undef vars io_wait_retrace, l20c9. (In future use CA65 to check..)
;  PET version working..
;-> ORAC508.S (3.6.24)
;  Imp USE_C16, C16/+4 version. Part Working..
;  C16 Hires (USE_PETSCII = 0) not working (init_graphics)..
;-> ORAC509.S (4.6.24)
;  Imp hires for c16/+4. Imp CUSTOMCHR index 1st chr in pieceset. (NOTE: C16 does not store rvs chars, only 1k set)
;-> ORAC510.S (11.6.24)
;  Imp constants pVDU_MOVE, pVDU_SIDE, pVDU_INPUT, pVDU_MSG, etc for VDU display msg areas..
;-> ORAC511.S (14.6.24)
;  Imp pVDU_CLOCKTOP, pVDU_INPUT, pVDU_IQ, pVDU_MOVE, pVDU_SIDE, pVDU_MSG etc pointers to msg areas on vdu.
;  Imp NEW_FRONTEND, and new more flexible versions of draw1piece, draw_calcpos, drawpieces, draw_board..
;  Imp IS_V20 simple VIC20+8K version. PETSCII only.
;  Imp smaller frontscr help text for the VIC
;  Bug in numberbrd..
;-> ORAC512.S (14.6.24)
;  Minor bits.. Experiment with otestX.s for VIC screen options. IS mem_page2 correct for all setups?
;-> ORAC513.S (17.6.24)
;  More tidying of old patches, removing excess code.
;  Imp IS_V20_UNEXPANDED to compile for unexpanded VIC (orac.prg =3270 bytes). Works with minimal setup..
;    (NOTE - tableweight,tmancent could fit in V20 colour ram, uses low nibble only.)
;-> ORAC514.S (18.6.24)
;  (NOTE: use: ./hexdump.elf chkpix4.bmp B 54,9999 >chkpix4.s  to conv bmp to asm, see test dir)
;  Imp hires chr set for VIC.
;  Imp V20_RELOC when 8k+ to move program to $1C00 to allow custom chr chrset at $1400. (Note use: LOAD"ORAC.PRG",8)
;  Imp USE_HIRES rather than USE_PETSCII.. (then USE_HIRES=1,2,3 for different piece sets, =0 for PETSCII)
;-> ORAC515.S (20.6.24)
;  Add more piece sets.
;-> ORAC516.S (21.6.24)
;  Bugs about draw_border etc.. Tidy up before adding more features..
;  Version with all machines working.
;-> ORAC517.S (21.6.24)
;  Imp USE_COLOR and color_table  ; Foreground, Background, White pieces, Black pieces
;  Imp color for pieces (use VDU_COLOR_EOR). Imp F1/F3/F5/F7 to select.
;  Imp COLOR_TRANSLATE & io_color_translate for C16: Translate color from index to actual colour to use.
;-> ORAC518.S (24.6.24)
;  imp USE_SOUND, io_sound etc.
;-> ORAC519.S (25.6.24)
;  Imp larger 2x3 piece sets so they center properly etc. 
;  Imp V command to toggle use of colored white graphics as black.
;-> ORAC520.S (25.6.24)
;  USE_CURSOR.. Get simple working version, cursor & space selects moves..
;-> ORAC521.S (25.6.24)
;  Small bugs n bits. Call draw_board_only after cursor move input
;  Imp help_screen (H cmd).
;  Imp USE_RANDMODE, rand_mode.. Randomise play by scrambling low bits of eval. M# to set rand factor.
;-> ORAC522.S (27.6.24)
;  Adjust centering (V20_HPOS, V20_VPOS) for NTSC. 
;  Minor tweaks to compile with CA65.
;-> ORAC523.S (29.6.24)
; 
; Compiling ORAC with my ASM502:   
;    asm502 orac.s orac.prg >orac.lst    
; OR Compiling ORAC with CA65:     
;    ca65 -D USE_CA65=1 orac.s
;    cl65 -t none -o orac.prg orac.o    # If you set the "-t" computer type, you dont need the "-D USE_CA65=1" fixup
;========================================================================================================================

				; Set ONE of the following options
.define IS_C64 1		; CBM64
.define IS_PET 0		; PET Basic 2 (2001/30xx)
.define IS_C16 0		; CBM16/+4
.define IS_V20 0		; VIC20 8K+
.define IS_TXT 0		; Simple text I/O, NOT WORKING YET

.define IS_V20_UNEXPANDED 0	; Set for unexpanded 5K VIC20. (BUT ALSO SET IS_VIC)
.define V20_RELOC 0		; Set to make 8k VIC ver relocate to $1C00.. (needed for hires, otherwise petscii)
 
.define USE_HIRES    1		; Set to 1,2,3.. to use different Hi-res UDG chrsets, =0 for PETSCII (Needed for PET). 1=c64/c16, 2=v20..
.define SMALLER_CODE 0		; Set this to take out some routines (Title screen, Brd Save/load etc)
.define SHOW_MOVE_INFO 1	; Set to add code to show best move, eval, node count etc..
.define USE_CLOCK    1		; Set to add chess style clock display.
.define NEW_FRONTEND 0		; Use more modern frontend (=2 small, info underneath) (NEEDED for VIC20).. (0 for PET/C16/C64)
.define USE_COLOR    1		; Set to use color graphics (0 for PET)
.define USE_SOUND    1		; Enable simple beep sound on move (=1 click, =2 long beep)
.define USE_CURSOR   1		; Allow use of cursor+space to select moves.
.define USE_RANDMODE 1		; Use randomisation to improve play variety (M0..M15 set random factor)
.define SET_HEIGHT   3		; Height of hires piece set in chars (2x2 or 2x3)..

.define VERSION1 $30+ 5		; ORAC VERSION as ascii digits
.define VERSION2 $30+ 2
.define VERSION3 $30+ 3

UDG_CHARS = 102			; Extra UDG chars if USE_HIRES set
NO_UDG_CHR = 4

.if IS_C64			; CBM-64 SPECIFIC VARIABLES
    prgstart = $0801
    pVDU = $0400		; Ptr to 40x25 vdu..

    VDUX=40			; VDU size
    VDUY=25
    SQRX = 4			; Size of squares in chrs
    SQRY = 3
    .if NEW_FRONTEND		; Smaller board..
      SQRX = 2			
      SQRY = 2
    .endif

    zpStart  = $03		; Start zp storage
    zpEnd = $8f			; End zp storage

    CHR_HLINE = $77	; Horizontal line chr
    
    MAX_PLY = 24		; Size of ply-stack tables
    MAX_MOVESTACK = 96		; Max moves on stack
	; was MAX_MOVESTACK = 128, MAX_PLY = 30	
    MAX_SPOTTAB = 48
    MAX_TAB = 24
    
	; Assort C64 specific locations (5/24:**** Sort/check for other targets **** ) 
	
	; Some low mem storage reclaim..
    mem_tape  = $033c		; mem (tape buffer $33c-3fb)
    mem_input = $0200		; mem (BUF: System Keyboard input buffer, $200-250)
    mem_stack = $0110		; mem (low stack)
    mem_page2 = $02a7		; mem (tape buf1: $2a7-329) 

    shiftkeyflag = $028d	; Set if SHIFT key down.. (653)
    key_buffer_size = $c6	; #chars in keybuf, Used by A autoplay cmd..

	; misc system vectors
    ;vectbasicerror = $0300
    vectwarmstart  = $0302
    vectirq        = $0314		; IRQ vector on c64 (points to $ea31, see oldirq)
    vectnmi        = $0318
    VICmem  = $d018		; VIC Memory Control Register: Bit:4-7 (VDU Base Bit 13-10) Bit:1-3 (Char Set Bit 13-11)
    C64memctl = 1		; C64 Memory Control reg.
    PTR_CHRSET = $3000		; chrset for UDG
    PTR_ROMSET = $d000
    CUSTOMCHR = $c0		; Offset for first custom char in chrset..

    CHR_TOPLEFT = $4f
    CHR_TOPRIGHT = $50
    CHR_LINERIGHT = $67
    CHR_CURSOR = 81
    
    vicborder = $d020
    vicback   = $d021
    c128fast  = $d030
    sid       = $d400
    pcolor    = $d800		; ptr to color ram
    VDU_COLOR_EOR = $dc		; EOR hi byte of loc Color ram ($d800) and VDU ($0400).. (Switch ptr COL/VDU)
    COLOR_TRANSLATE = 0

	; misc ROM vectors/routines
    ;romrestorevectors = $fd15
    romirq =$ea31			; Original c64 IRQ vector (at $0314)
    
    zp_jiffy_time = $a0		; cbm system clock, 3 bytes hi,med,low. in jiffys (1/60 sec)
    zp_jiffy_low  = $a2
    ;  .define C128_FAST 0		; Enables C128 fast mode..
.endif			; IS_C64


.if IS_PET			; PET SPECIFIC VARIABLES
    prgstart = $0401
    pVDU = $8000		; Ptr to 40x25 vdu..
    
    VDUX=40			; VDU size
    VDUY=25
    SQRX = 4			; Size of squares in chrs
    SQRY = 3
    .if NEW_FRONTEND		; Smaller board..
      SQRX = 2			
      SQRY = 2
    .endif

    zpStart  = $03
    zpEnd = $8f			; End zp storage

    CHR_HLINE = $63
    MAX_PLY = 24		; Size of ply-stack tables
    MAX_MOVESTACK = 96
    MAX_SPOTTAB = 48
    MAX_TAB = 24

    CHR_TOPLEFT = $4f
    CHR_TOPRIGHT = $50
    CHR_LINERIGHT = $67
    CHR_CURSOR = 81
    
    mem_tape  = $033a		; mem (tape buffer 2: $33a-3f9)
    mem_input = $0200		; mem (BUF: System Keyboard input buffer, $200-250)
    mem_stack = $0110		; mem (low stack $110 )
    mem_page2 = $027a		; mem (tape buffer 1: $27a-329)

    shiftkeyflag = 152		; Set if shift key down (peek(152))
    key_buffer_size = $9e	; #chars in keybuf, Used by A autoplay  cmd..
    
    ; vectwarmstart  = $0302
    vectirq        = $0090		; IRQ vector on c64 (points to $ea31, see oldirq)
    vectnmi        = $0094

	; Assort PET specific locations 
    ;petpia1keysel = $e810		; Bits 0-3 sel key row. PET 59408
    ;petpia1keyin  = $e812
    ;petrand1 = $e844			; PET 59460
    ;petrand2 = $e845
    
    PET_SOUND_e84a = $e84a
    PET_SOUND_e848 = $e848
    PET_SOUND_e84b = $e84b			; Turn off sound
    PET_CASE = $e84c			; PET:poke 59468,12/14 low/up case.

    romirq =$e62e			; Original IRQ routine

    zp_jiffy_time = $8d		; cbm jiffy clock, 3 bytes hi,med,low. 1/60 sec.
    zp_jiffy_low  = $8f
    
.endif			; IS_PET


.if IS_C16			; CBM-16 / PLUS4 SPECIFIC VARIABLES
    prgstart = $1001
    pVDU = $0C00		; Ptr to 40x25 vdu..
    
    VDUX=40			; VDU size
    VDUY=25
    SQRX = 4			; Size of squares in chrs
    SQRY = 3
    .if NEW_FRONTEND		; Smaller board..
      SQRX = 2			
      SQRY = 2
    .endif

    zpStart  = $03
    zpEnd = $8f			; End zp storage

    CHR_HLINE = $77
    CHR_LINERIGHT = $67
    
    .if USE_HIRES
      CHR_TOPLEFT = $68
      CHR_TOPRIGHT = $69
      CHR_CURSOR = 42
    .else
      CHR_TOPLEFT = $4f
      CHR_TOPRIGHT = $50
      CHR_CURSOR = 81
    .endif
    
    MAX_PLY = 24		; Size of ply-stack tables
    MAX_MOVESTACK = 96		; Max moves on stack
	; was MAX_MOVESTACK = 128, MAX_PLY = 30	
    MAX_SPOTTAB = 48
    MAX_TAB = 24
    
	; Some low mem storage reclaim..
    mem_tape  = $0332		; mem (tape buffer $332-)
    mem_input = $0200		; mem (BUF: System Keyboard input buffer, $200-250)
    mem_stack = $0110		; mem (low stack)
    mem_page2 = $025e		; mem (2nd page DOS/GFX work area) 

    shiftkeyflag = $0543	; Set if SHIFT key down (dec 1347)
    key_buffer_size = $ef	; keyboard queue index, Used by A autoplay cmd.. (dec 239)
    
	; misc system vectors
    ;vectbasicerror = $0300
    vectwarmstart  = $0302
    vectirq        = $0314		; IRQ vector on c64 (points to $ea31, see oldirq)
    vectnmi        = $0318
    C64memctl = 1		; C64 Memory Control reg.
    PTR_CHRSET = $3000		; chrset for UDG
    PTR_ROMSET = $d000
    CUSTOMCHR = $3f		; Offset for first custom char in chrset..
    
				; C16/+4 TED registers
    TED_VOICE1LO = $FF0E	; Low byte freq voice 1
    TED_VOICE2LO = $FF0F	; Low byte freq voice 2
    TED_VOICE2HI = $FF10	; bits 0-1 High bits freq for voice 2
    TED_VOL      = $FF11	; bit0-3:   Volume, bit4: Select voice 1, bit5:tone gen voice 2, bit6: noise gen voice 2.
    TED_MEMCTL   = $FF12	; Bitmap/Character Memory Control (Bit0-1:Sound Voice 1MSB, Bit 2:chr/bmp ROM/RAM, Bit3-5:Bmp Base 8K steps)
    TED_MEMCTL_TEXT = 196	; Bit 2 on for Text (use ROM)
    TED_MEMCTL_GFX  = 192	; Bit 2 off for GFX (use RAM)
    TED_CHRBASE = $FF13	  	; System Clock and Character Memory Control (Bit 2-7:chr base 1K steps)
    TED_VDULOC  = $FF14	  	; Video ram loc 2k steps (Bit 3-7)
    TED_VDUCOL  = $FF15	  	; Screen background colour (Bit 0-3:Color, Bit 4-6:Luminance)
    TED_BORDER  = $FF19	  	; Border colour
    TED_RASTHI  = $FF1C	  	; Raster line hi bit (Bit 0)
    TED_RASTER  = $FF1D	  	; Current raster line (Bit 0-7) 
    
    ;TEDromselect = $ff3e  ; Write ANY VAL here to Select ROM
    ;TEDramselect = $ff3f  ; Write ANY VAL here to Select RAM
    ;TEDromlatch  = $fdd0  ; Write ANY VAL here to select lorom=BASIC hirom=Kernal (fdd0-fddf selects various basic/kernal/other rom combos)
    

    pcolor    = $0800		; ptr to color ram
    VDU_COLOR_EOR = $04		; EOR hi byte of loc Color ram ($0800) and VDU ($0C00).. (Switch ptr COL/VDU)
    COLOR_TRANSLATE = 1

	; misc ROM vectors/routines
    romirq =$ce0e			; Original c16 IRQ vector (at $0314)
    
    zp_jiffy_time = $a3		; cbm system clock, 3 bytes hi,med,low. in jiffys (1/60 sec).
    zp_jiffy_low  = $a5

.endif			; IS_C16

.if IS_V20		; VIC20 SPECIFIC VARIABLES
			; NOTE - set IS_V20_UNEXPANDED for 5k vic..
			;  ..set V20_RELOC for 8k+ with gfx (prgstart=$1C01), then fix prg start bytes (or use LOAD"ORAC.PRG",8)
    pVDU = $1000		; Ptr to vdu..
 
    .if V20_RELOC		; Code to relocate program from $1200->$1C00 (for 8k+ using hires gfx, chrset at $1400)
      prgstart = $1c01
    .else
      prgstart = $1201		; For 8k+ VIC
    .endif
    
    .if USE_HIRES
      VDUX = 24			; VDU size
      VDUY = 25
    .else
     VDUX = 22			; VDU size
     VDUY = 23
    .endif
    SQRX = 2			; Size of squares in chrs
    SQRY = 3

    zpStart  = $03		; Start zp storage
    zpEnd = $8f			; End zp storage

    CHR_HLINE = $63	; Horizontal line chr
    
    CHR_TOPLEFT = $4f
    CHR_TOPRIGHT = $50
    CHR_LINERIGHT = $67
    CHR_CURSOR = 81

    MAX_PLY = 24		; Size of ply-stack tables
    MAX_MOVESTACK = 96		; Max moves on stack
    MAX_SPOTTAB = 48
    MAX_TAB = 24
    
	; Assort VIC20 specific locations 
	
	; Some low mem storage reclaim..
    mem_tape  = $033c		; mem (tape buffer $33c-3fb)
    mem_stack = $0110		; mem (low stack)
    mem_input = $0200		; mem (BUF: System Keyboard input buffer, $200-250)
    mem_page2 = $02a7		; mem (misc:$2a7-314) 

    shiftkeyflag = $028d	; Set if SHIFT key down.. (653)
    key_buffer_size = $c6	; #chars in keybuf, Used by A autoplay cmd..

	; misc system vectors
    ;vectbasicerror = $0300
    vectwarmstart  = $0302
    vectirq        = $0314		; IRQ vector on c64 (points to $eabf, see oldirq)
    vectnmi        = $0318
    CUSTOMCHR = $c0		; Offset for first custom char in chrset..

    V20_HPOS    = $9000		; bits 0-6 horizontal pos/centering, bit 7 sets interlace scan (36864) [PAL:12,NTSC:5]
    V20_VPOS    = $9001		; vert position [PAL:38,NTSC:25]
    V20_HSIZE   = $9002		; bits 0-6 set # of columns, bit 7 part video matrix address
    V20_VSIZE   = $9003		; bits 1-6 set # of rows, bit 0 for 8x8 or 8x16 chars
    V20_RASTER  = $9004		; current video raster line
    V20_VIDADD  = $9005		; Vid/chrset address: bits 0-3 charset address, bits 4-7 screen address.  (36869)
    V20_PENH    = $9006		; Light pen horiz
    V20_PENV    = $9007		; Light pen vert
    V20_PADX    = $9008		; Paddle X
    V20_PADY    = $9009		; Paddle Y
    V20_VOICE1  = $900a		; Voice osc 1 (128-255) bit 7 on/off (36874)
    V20_VOICE2  = $900b		; Voice osc 2 (128-255) bit 7 on/off
    V20_VOICE3  = $900c		; Voice osc 3 (128-255) bit 7 on/off
    V20_NOISE   = $900d		; Voice noise (128-255) bit 7 on/off
    V20_VOL     = $900e		; bit 0-3 overall volume, bits 4-7 aux color in multicolor mode.
    V20_BACKCOL = $900f		; Screen/border color, bits 4-7 screen background, bits 0-2 border, bit 3 for inverted/normal mode (36879)
    V20_VIA1    = $9110		; VIC20 6522 VIA1
    V20_VIA2    = $9120		; VIC20 6522 VIA2
    V20_COLRAM  = $9400		; VIC20 colour ram (9600 for vdu $1e00)  
    V20_CHARSET = $8000		; VIC20 ROM char set.
				; VIEW: 1 for x=36864 to 36879:print peek(x),:next
    PTR_CHRSET = $1400		; chrset for UDG
    PTR_ROMSET = $8000
    pcolor    = V20_COLRAM		; ptr to color ram (or 9600)
    VDU_COLOR_EOR = $84		; EOR hi byte of loc Color ram ($9400) and VDU ($1000).. (Switch ptr COL/VDU)
    COLOR_TRANSLATE = 0
    
	; misc ROM vectors/routines
    romirq =$eabf			; Original VIC20 IRQ vector (at $0314)
    
    zp_jiffy_time = $a0		; cbm system clock, 3 bytes hi,med,low. in jiffys (1/60 sec)
    zp_jiffy_low  = $a2

.endif			; IS_V20

.if IS_V20_UNEXPANDED	; Unexpanded VIC
    prgstart = $1001
    pVDU = $1e00		; Ptr to vdu..
    MAX_PLY = 20		; Size of ply-stack tables
    MAX_MOVESTACK = 80		; Max moves on stack
    VDUX = 22			; VDU size
    VDUY = 23
    SQRX = 2
    SQRY = 2
.endif


	; Calc Assort VDU display area constants.. (for old frontend/display)
	; These appear on the right of the board..
	
pVDU_INFO = pVDU + SQRX * 8		; Put info to right of board
pVDU_CLOCKTOP= pVDU_INFO + VDUX*4  	; VDU Clock TOP 		(pVDU+$c0 - originally)
pVDU_CLOCKWH = pVDU_INFO + VDUX*6  + 1	; VDU White Clock 		(pVDU+$111)
pVDU_CLOCKBL = pVDU_INFO + VDUX*9  + 1	; VDU Black Clock 		(pVDU+$189)
pVDU_IQ      = pVDU_INFO + VDUX*12 + 5	; VDU IQ level 			(pVDU+$205)
pVDU_MOVE    = pVDU_INFO + VDUX*18 + 1	; VDU computer move display 	(pVDU+$2f1)
pVDU_SIDE    = pVDU_INFO + VDUX*21 + 1	; VDU Side to move (W#/B#)	(pVDU+$369)
pVDU_INPUT   = pVDU_INFO + VDUX*22 + 1	; Input buffer on screen. 	(pVDU+$391)
pVDU_MSG     = pVDU_INFO + VDUX*23 + 1	; VDU msg display 		(pVDU+$3b9)
pVDU_BESTMOV = pVDU_INFO + VDUX*19 + 1	; VDU best move 		(pVDU+793)
pVDU_BESTVAL = pVDU_INFO + VDUX*20 + 1	; VDU best eval 		(pVDU+833)
pVDU_NODES   = pVDU_INFO + VDUX*24 + 1	; VDU node count 		(pVDU+994)

.if NEW_FRONTEND=2		; ALT Put info under display
  SQRX = 2			; Smaller board..
  SQRY = 2
  pVDU_INFO = pVDU + VDUX*17		; Put info under display
  pVDU_INPUT = pVDU_INFO		; Input buffer on screen.
  pVDU_MSG  = pVDU_INFO + 8		; VDU msg display 
  pVDU_IQ   = pVDU_INFO +VDUX		; VDU IQ level  
  pVDU_SIDE = pVDU_INFO +VDUX +4	; VDU Side to move (W#/B#)
  pVDU_MOVE = pVDU_SIDE + 3	  	; VDU computer move display 

  pVDU_CLOCKWH = pVDU_INFO +VDUX*2 	; VDU White Clock 
  pVDU_CLOCKBL = pVDU_CLOCKWH + 8	; VDU Black Clock 
  pVDU_BESTMOV = pVDU_CLOCKWH + VDUX	; VDU best move 		(pVDU+793)
  pVDU_BESTVAL = pVDU_BESTMOV+5		; VDU best eval 		(pVDU+833)
  pVDU_NODES   = pVDU_BESTMOV+10	; VDU node count 		(pVDU+994)
.endif

MAX_BRD = 64

.if (USE_CA65 = 1)		; Kludge to gen start address for CA65 (CmdLine must have "-D USE_CA65=1")
 .ORG prgstart-2
 .WORD prgstart
.else
 .ORG prgstart
.endif


	; Zero page vars

  tboard = $03		; Put Board area in zero page ($03-$42)

	;     Board translation to real sqrs (mirrored left/right)

	;+00:  29 -- 30 -- 31 -- 32 --
	;+08:  -- 25 -- 26 -- 27 -- 28
	;+10:  21 -- 22 -- 23 -- 24 --
	;+18:  -- 17 -- 18 -- 19 -- 20
	;+20:  13 -- 14 -- 15 -- 16 --
	;+28:  -- 09 -- 10 -- 11 -- 12
	;+30:  05 -- 06 -- 07 -- 08 --
	;+38   -- 01 -- 02 -- 03 -- 04

	; (blackman=$fe, whiteman=$02)

  zPtrGen  = $54	; Generic vdu pointer used by draw, coord calc, brd number etc. (PET:$54,$55)
  zMem1    = $56	; Used by newbrd, humanmove, draw 
  zMem2    = $57
  zMem3    = $58
  zMem4    = $59	
  zMem5    = $5a
  zMem6    = $5b
  zEvalTot = $5e	; Eval total used by search
  ztemp    = $60
  zPtrDir  = $61	; Used to point to move-list dir table during search (>256, so 16bit ptr needed)
  zmove    = $63	; (PET:V5)
  zPtrMov  = $64	; Used to point to move-list table during search (>256, so 16bit ptr needed)
  zMovOrdInd = $66
  ; mgenflag  = $67	; unused
  zMem7     = $68
  zEvalTemp = $69

  oldpiece = $88
  oldtaken = $89
  oldtime  = $8a
  oldblack = $8b
  oldwhite = $8c
.if SHOW_MOVE_INFO		; Node count vars
  nodecount = $fd		; 3 byte node count ($8d-$8f)
.endif

	; Vars moved into zero page mem

  side2move =$43
  upperply  =$44       		; Max ply depth (PET:VU, 03D0)
  lowerply  =$45
  interest  =$46
  iqcut     =$47		; IQ setting 
  alphakill =$48		; Kill further ab lookaheads (PET:VA=03d3)		
  oldkings  =$49		; (OK, 03E0)
  curcolor  =$4a		; Side to move (VC)
  valwhite  =$4b		; Val of white pieces (VW, 03E3) (2 for man, 3 for k)
  valblack  =$4c		; Val of black pieces (VB)
  nkings    =$4d		; No of kings (NK)
  mvkiller  =$4e		; VK
  jumpflag  =$4f		; move is jump (JP, 03E7)
  cmove     =$50		; VM, 03E8
  cply      =$51		; CP, 03E9
  mvoption  =$52		; VO
  nmoves    =$53		; No of moves (VN)

  cmobil    =$6b		; MO, 03EC
  dirvert   =$6c		; DV=$03ed  
  dirhoriz  =$6d		; DH=$03ee
  ctime     =$6e		; VT
  bestmove  =$6f		; Best move.  (BM, 03F0)
  bestdir   =$70		; Best move dir.  (BD)
  bestval   =$71		; Best move eval (BV, 03F2)
  mvgenindex=$73		; V3, 03F4
  takeflag  =$74		; V4
  ckillmove =$75		; KX
  ckilldir  =$76		; KY
  mvgenflag =$77		; VF, 03F8

	; CBM Kernal system routines
  kernopen   = $ffc0		; Open a logical file
  kernclose  = $ffc3		; Close file..
  kernchkout = $ffc9		; Sel handle for chrout
  kernclrchn = $ffcc
  kernchrout = $ffd2		; Print chr (A) to output 
  kerngetin  = $ffe4		; Get char (A) from input

	; piece vals on board
pwhiteman  = $02
pwhiteking = $03
pblackman  = $fe
pblackking = $fd

;---------------------------------------------------------------------------
;  BASIC HEADER START for each machine type, does: jmp runprog

.if IS_C64
   ; BASIC START  - 6502 sys 2062
  .byte $0c,$08,$66,$19,$9e,$20,$32,$30,$36,$32,$00,$00,$00
.endif

.if IS_PET
   ; BASIC START - 6502 sys 1038
  .byte $0c,$04,$66,$19,$9e,$20,$31,$30,$33,$38,$00,$00,$00
.endif

.if IS_C16
   ; BASIC START  - 6502 sys 4110
  .byte $0c,$10,$66,$19,$9e,$20,$34,$31,$31,$30,$00,$00,$00
.endif

.if IS_V20
 .if IS_V20_UNEXPANDED			; Unexpanded VIC20 5K..
      ; BASIC START:  6502 sys 4110
   .byte $0c,$10,$66,$19,$9e,$20,$34,$31,$31,$30,$00,$00,$00
   
 .else					; expanded VIC20 8K+.. (start $1201)
      ; BASIC START:  6502 sys 4622
   .byte $0c,$12,$66,$19,$9e,$20,$34,$36,$32,$32,$00,$00,$00
   
   .if V20_RELOC		; Code to relocate program from $1200->$1C00 (NEEDED for 8K+ with hires)
				; (NOTE this moving-code compiles to $1C0E, but actually runs at $120E..)
      ldx #(>(prglen))+1		; #pages to move..
      ldy #0
reloc1:      			; Copy from top to bottom..
      lda $1200 + (prglen & $ff00),y
      sta $1c00 + (prglen & $ff00),y
      iny
      bne reloc1
      dec reloc1-$0a00+2	; Decrement hi byte of lda/sta pointers.. (self mod code) (offset by $0a00)
      dec reloc1-$0a00+5
      dex
      bpl reloc1
				; Code should now be moved to right place ($1c00), drop thro to execute as normal..
   .endif	; (V20_RELOC)
   
 .endif		; (IS_V20_UNEXPANDED)
.endif		; (IS_V20)

absstart:
  jmp runprog

.if USE_CLOCK
clockwhite:
.byte $30,$30,$30,$30,$30,$30
clockblack:
.byte $30,$30,$30,$30,$30,$30
.endif

color_table:		; COLOR TABLE: Foreground, Background, White pieces, Black pieces (CTRL-D..CTRL-G)
color_ink:
  .byte 3
color_back:
  .byte 0
color_white:
  .byte 1
color_black:
  .byte 4
color_temp:		; temp for color..
 .byte 0

graphics_mode:		; Select graphics mode.. (0=normal, 8=use colored white graphics for black. V command toggles)
 .byte 8

.if USE_CURSOR
cursor_mode:		; Set for cursor input mode..
 .byte 0
cursor_pos:		; Set for cursor input mode..
 .byte 1
cursor_chr:		; Old chr
 .byte 32
.endif

.if USE_RANDMODE
rand_mode:
  .byte 1			; Set to allow randomisation of moves	
rand_val:
  .byte 0			; Contains val to EOR with eval (zEvalTot) changes every 1/60 sec..
ROM_RAND = $fc00		; Page of ROM to use to help randomisation..
.endif

;---------------------------------------------------------------------------
; Board save/load routines..

.if SMALLER_CODE

load_board_cmd:		; Patch Called by L command (PET:$0680)
save_board_cmd:		; Patch Called by S command (PET:$0689)
  rts

.else

brdstore:		; Save/load area for 8 brds (0-7, 32 bytes each) (PET $0500-$05ff)

 .byte $00,$00,$00,$00,$00,$00,$fe,$00,$00,$00,$00,$00,$00,$00,$00,$fe		; Brd 0 (L0/S0)
 .byte $00,$00,$00,$00,$00,$00,$fe,$00,$00,$00,$00,$02,$00,$00,$00,$02
 .byte $02,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00		; Brd 1 (L1/S1)
 .byte $00,$00,$00,$00,$00,$fe,$00,$00,$00,$00,$fe,$03,$00,$02,$00,$00
 .byte $00,$00,$00,$00,$00,$00,$03,$00,$00,$00,$00,$00,$00,$fd,$00,$00		; Brd 2
 .byte $00,$00,$03,$00,$00,$fd,$00,$03,$00,$00,$00,$00,$00,$00,$00,$00
 .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00		; Brd 3
 .byte $00,$00,$00,$00,$fe,$00,$00,$fe,$02,$00,$00,$00,$fe,$00,$03,$02
 .byte $00,$fe,$00,$fe,$00,$00,$02,$fe,$00,$00,$02,$00,$fe,$fd,$02,$02		; Brd 4
 .byte $00,$00,$02,$02,$fe,$00,$02,$02,$00,$fd,$fe,$02,$00,$00,$00,$00
 .byte $00,$00,$03,$00,$fe,$00,$00,$00,$00,$00,$00,$02,$00,$00,$00,$02		; Brd 5
 .byte $00,$00,$03,$fd,$00,$00,$02,$00,$00,$fe,$00,$02,$fd,$fd,$00,$00
 .byte $00,$00,$00,$00,$00,$02,$00,$00,$00,$00,$00,$fe,$00,$00,$02,$00		; Brd 6
 .byte $00,$00,$fe,$fe,$00,$fe,$fe,$fe,$00,$02,$00,$00,$00,$00,$02,$02
 .byte $02,$00,$fe,$00,$fe,$00,$00,$00,$00,$00,$02,$00,$02,$00,$02,$fd		; Brd 7
 .byte $02,$fd,$00,$00,$02,$00,$00,$00,$02,$fd,$fe,$02,$03,$00,$00,$00


zpBrds = zMem1

read_cmd_val:   		; Read board # (0-7) from cmd line, change to brdstore ptr (zpBrds)
  lda pVDU_INPUT+1
  and #$07
  asl                   ; Mult by 32.:
  asl
  asl
  asl
  asl
  adc #<(brdstore)
  sta zpBrds
  lda #>(brdstore)
  adc #0
  sta zpBrds+1
  rts

save_board_cmd:   		; Patch Called by S command (PET:$0689)
  jsr read_cmd_val
savebrd:				; Save brd (PET:S0=$0ab6, page i10)
  ldy #$20
saveb1:
  sty zPtrGen+1
  jsr conv2coord
  tax
  ldy zPtrGen+1
  lda tboard,x
  sta (zpBrds),y
  dey
  bne saveb1
  rts

load_board_cmd:                  ; Patch Called by L command (PET:$0680)
  jsr read_cmd_val
loadbrd:			; Load brd (PET:S2=$0acf)
  ldy #$20
loads3:
  sty zPtrGen+1
  jsr conv2coord
  tax
  ldy zPtrGen+1
  lda (zpBrds),y
  sta tboard,x
  dey
  bne loads3
  rts

.endif			; SMALLER_CODE

patchnumberbrd:   		; CMD:N Patch called by N cmd  (PET:$06a0)
  jsr numberbrd
  pla
  pla
  jmp humancmd

;---------------------------------------------------------------------------
; oracbrain:- Search recursively for a move..

		; New version (tableweight) calculated properly, can be anywhere.

oracbrain:             ; $06c0  CALCULATE A MOVE

selmancent:		; Mancent table selector - sets up table at (tmancent)
  ldx #$60		; Sel (tableweight+$60) if kings on board
  lda nkings
  bne selman1
  lda valwhite
  clc
  adc valblack
  ldx #$00		; Select (tableweight+$00) if material >41, ie 21 men or more.:
  cmp #$29
  bcs selman1
  ldx #$20		; Select (tableweight+$20) if material >35, ie 18 men or more.:
  cmp #$23
  bcs selman1
  ldx #$40		; Select (tableweight+$40) if material <36, ie 17 men or less.:

selman1:
  txa			; Calc address to get weight table from: ptr (zEvalTot) = tmanweight + x
  clc
  adc #<tableweight	
  sta zEvalTot
  ldx #>tableweight	
  bcc selman2
  inx
selman2:
  stx zEvalTot+1

  ldy #$1f		; Move 32 bytes (sqr order) from (zEvalTot) into 64 byte board at (tmancent)
selman3:
  tya                   ; Translate (Y) 0..31 to (X) 0..63 TBOARD co-cord
  pha
  lsr
  lsr
  lsr
  pla
  rol
  tax
  lda (zEvalTot),y
  sta tmancent,x
  dey
  bpl selman3

  jmp findbestalpha


tableweight:			; Phase/Weight tables (PET:$0700) Best on a page boundary.
.byte $02,$08,$08,$08,$03,$05,$07,$07,$04,$05,$07,$06,$00,$07,$06
.byte $05,$05,$01,$03,$02,$00,$00,$00,$00,$07,$09,$09,$07,$0e,$0e
.byte $0c,$0a,$00,$06,$06,$06,$01,$03,$06,$05,$03,$04,$06,$04,$00
.byte $07,$05,$04,$05,$05,$07,$04,$03,$05,$05,$02,$09,$0b,$0b,$09
.byte $0e,$0e,$0c,$0a,$00,$04,$04,$04,$01,$02,$04,$04,$02,$03,$05
.byte $03,$01,$06,$05,$04,$05,$08,$08,$05,$07,$0a,$0a,$04,$0b,$0d
.byte $0d,$0b,$0e,$0e,$0c,$0a,$01,$03,$03,$03,$00,$00,$02,$04,$04
.byte $02,$04,$03,$06,$05,$03,$06,$07,$06,$06,$05,$08,$09,$09,$06
.byte $0a,$0b,$0b,$09,$0e,$0e,$0c,$0a

tkingcent:		; Weight table for kings..
.byte $0a,$00,$0c,$00,$0c,$00,$0e,$00
.byte $00,$0f,$00,$10,$00,$11,$00,$0e
.byte $0c,$00,$12,$00,$14,$00,$11,$00
.byte $00,$10,$00,$15,$00,$14,$00,$0c
.byte $0c,$00,$14,$00,$15,$00,$10,$00
.byte $00,$11,$00,$14,$00,$12,$00,$0c
.byte $0e,$00,$11,$00,$10,$00,$0f,$00
.byte $00,$0e,$00,$0c,$00,$0c,$00,$0a

tmancent:		; Weight table for men..
.byte $02,$00,$08,$00,$08,$00,$08,$00
.byte $00,$03,$00,$05,$00,$07,$00,$07
.byte $04,$00,$05,$00,$07,$00,$06,$00
.byte $00,$00,$00,$07,$00,$06,$00,$05
.byte $05,$00,$01,$00,$03,$00,$02,$00
.byte $00,$00,$00,$00,$00,$00,$00,$00
.byte $07,$00,$09,$00,$09,$00,$07,$00
.byte $00,$0e,$00,$0e,$00,$0c,$00,$0a

;---------------------------------------------------------------------------

inputcmd:			; Input human command      (PET:I0=$0823)
  jsr input_clear		; Clr input buffer (on vdu)
				; (x=$ff, but next will increment it to 0)
input_next:			
  inx 				; Next char
input_loop:			; Reloop point..
  lda #$a0			; Display cursor
  sta pVDU_INPUT,x 
input6:
  jsr input_getk

.if USE_CURSOR 
  jsr cursor_input		; Test for cursor input of moves
input_cont:  
.endif 			; (USE_CURSOR)

  and #$7f
  cmp #$0d			; Return hit?
  bne input2			
input5:
  lda #$20			; Clear cursor
  sta pVDU_INPUT,x 
  ; jsr io_fast_test		; Patch for C128 fast mode..
  rts				; All done, command returned at (pVDU_INPUT)
   
input2:
  cmp #$14			; Delete?
  bne input3
  txa 
  beq input_loop
  jsr input5			; clr cursor
  dex 
  clc 
  bcc input_loop		; Cont input..
  
input3:
  cmp #$21
  bcc input6			; <= space, skip..
  cpx #$06
  bcs input6			; Max reached, skip
  and #$3f			; Convert asc to petscii..
  sta pVDU_INPUT,x 
  clc 
  bcc input_next			; Jump to reloop
  
input_getk:
  txa
  pha 
  jsr io_getkey		; Get a key (kerngetin, $FFE4), null if none..
  tay 
  pla 
  tax 
  tya 
  rts

input_clear:
  lda #$20			; Clr input buffer (on vdu)
  ldx #$06
input1:
  sta pVDU_INPUT,x 
  dex 
  bpl input1
  rts
   
.if USE_CURSOR			; Allow input input with cursor/space..
  
cursor_tab:
  .byte 17,145,157,29		; Cursor down,up,left,right
cursor_dir:
  .byte $08,$f8,$ff,$01		; Direction for each char

cursor_input:			; Test for cursor input. (A=key hit)
  stx zMem3			; Preserve X
  ldy #3
cursor1:
  cmp #32
  beq cursor_select		; Select a move?
  cmp cursor_tab,y
  beq cursor_hit  
  dey
  bpl cursor1
  rts
  
cursor_hit:
  lda cursor_pos
  tax
  clc
  adc cursor_dir,y
  and #$3f
  sta cursor_pos
  jsr draw_calcpos	; Calc sqr cursor was on..
  ldy #0
  lda cursor_chr
  sta (zPtrGen),y
  ldx cursor_pos
  jsr draw_calcpos
  ldy #0
  lda (zPtrGen),y
  sta cursor_chr
  lda # CHR_CURSOR
  sta (zPtrGen),y
cursor2:
  ldx zMem3
  lda #0
  rts

cursor_select:		; Space hit, so try select cur sqr..
  lda cursor_pos
  eor #7		; Flip X position..
  sta zPtrGen		; Save brd pos of cursor (0-63), convert to sqr (1-32)
  jsr coord2brd
  lda zPtrGen+1
  jsr convbyte2asc	; Convert to ascii sqr number in (X:A)
  ldy zMem3
  bne cursor_select2	; Some input already, try sqr as 2nd selection..
  stx pVDU_INPUT	; Place 1st sqr ascii (1-32) in input buffer
  sta pVDU_INPUT+1
  ldx #2
  lda #0
  rts  

cursor_select2:		; 2nd sqr selected? if so, try move..
  cpy #2 
  bne cursor2
  stx pVDU_INPUT+3	; Place 2nd sqr ascii (1-32) in input buffer
  sta pVDU_INPUT+4
  lda #45		; Add '-'
  sta pVDU_INPUT+2
  jsr draw_board_only		; Draw pieces only..
  lda #13		; Supply return to execute move in input buffer.
  ldx #5	
  rts
  
		; ROUTINES USED: draw_calcpos, draw1piece, coord2brd, convbyte2asc
  
.endif 			; (USE_CURSOR)
    
;---------------------------------------------------------------------------

conv2coord:			; Convert Brd index to Coord. (PET:CB=$086d, Page i3)
  ldx zPtrGen+1
  dex
  txa
  lsr
  lsr
  sta zPtrGen
  asl
  asl
  asl
  asl
  sec
  sbc zPtrGen+1
  sec
  sbc zPtrGen+1
  adc zPtrGen
  sta zPtrGen
  txa
  lsr
  lsr
  lsr
  asl
  adc #$38
  sec
  sbc zPtrGen
  sta zPtrGen
  rts

coord2brd:				; Convert co-ord (in zPtrGen) to ascii board # (1-32 in zPtrGen+1) (PET:CC=$0890)
  lda zPtrGen
  and #$f8
  sta zPtrGen+1
  lda zPtrGen
  lsr 
  clc 
  adc #$1d
  sec 
  sbc zPtrGen+1
  sta zPtrGen+1
  rts 

convbyte2asc:			; Convert Acc to 2 digit ascii decimal 00-99(X;A)  (PET CA=$08a2, page i4)
  ldx #$2f
convb0:
  inx 
  sec 
  sbc #$0a
  bpl convb0
  clc 
  adc #$3a
  rts
   
convbyte2dec:			; Conv Acc to ascii dec (X;A) as above but with leading space.
  jsr convbyte2asc
  cpx #$30
  bne convb2
  ldx #$20
convb2:
  rts
   
convdec2by:			; Convert decimal # (NN) to Byte, SEC if bad in. (PET: CT=$08B8)
  jsr getuserin
  bcs convd3
  pha
  iny
  jsr getuserin
  tax 
  pla 
  bcs convd4
  asl 
  sta zPtrGen+1
  asl 
  asl 
  adc zPtrGen+1
  sta zPtrGen+1
  txa 
  iny 
  adc zPtrGen+1
convd4:
  sta zPtrGen+1
  clc 
convd3:
  rts
   
getuserin:				; Read/validate user input. (PET: CR=$08d7)
  lda pVDU_INPUT,y
  cmp #$3a
  bcs convd5
  sec
  sbc #$30
  sec
  sbc #$d0
  bcs convd5
  and #$0f
convd5:
  rts

;---------------------------------------------------------------------------

setupnewbrd:				; Set up a New board (PET: BS=$08e9, page i5)
  ldx #$3f
  ldy #$07
setup3:
  lda #$07
  sta zMem1
setup2:
  tya 
  clc 
  adc zMem1
  lsr 
  lda #$00
  bcs setup0
  cpy #$05
  bcc setup1
  ; ???
  lda # pblackman
setup1:
  cpy #$03
  bcs setup0
  lda # pwhiteman
setup0:
  sta tboard,x 
  dex 
  dec zMem1
  bpl setup2
  dey 
  bpl setup3
  rts
   
erasebrd:		; CMD:C   Erase whole brd (PET:BE=$0912)
  ldx #$3f
  lda #$00
erase1:
  sta tboard,x 
  dex 
  bpl erase1
  rts

;---------------------------------------------------------------------------

exchange_side:		; eXchange side to move (X command)
  lda side2move
  eor #$80
  sta side2move
  rts
  
;---------------------------------------------------------------------------

change_graphics:	; Change to alternate graphics (V command)
.if USE_COLOR
  lda graphics_mode
  eor #8
  sta graphics_mode
.endif  
  rts

set_rand_mode:
.if USE_RANDMODE
  jsr convdec2by
  sta rand_mode
.endif
  rts

;---------------------------------------------------------------------------
   
flipbrd:		; Flip board upside down (F command) (PET BX=$091d)
  ldx #$00
  ldy #$3f
exch1:
  lda #$00
  sec 
  sbc tboard,x 
  pha 
  lda #$00
  sec 
  sbc tboard,y 
  sta tboard,x 
  pla 
  sta tboard,y 
  dey 
  inx 
  cpx #$20
  bne exch1
  rts

;---------------------------------------------------------------------------
   
reflectbrd:		; Reflect board vertically (PET B4=$093c, page i6)
  ldx #$3f
reflectb6:
  txa 
  and #$04
  bne reflectb5
  txa 
  eor #$07
  tay 
  lda tboard,x 
  pha 
  lda tboard,y 
  sta tboard,x 
  pla 
  sta tboard,y 
reflectb5:
  dex 
  bpl reflectb6
  rts

;---------------------------------------------------------------------------
   
numberbrd:			; Put numbers on brd (PET:NB=$0959)
  ldx #$3f
  stx zMem2
number0:
  ldx zMem2
  stx zPtrGen
  jsr coord2brd
  tax 
  dex 
  txa 
  eor #$03
  tax 
  inx 
  txa 
  jsr convbyte2dec
  pha 
  txa 
  pha 
  ldx zMem2
  jsr draw_calcpos
  pla 
  tax 
  pla 
  ldy # VDUX
  bcc number1
  sta (zPtrGen),y
  txa 
  ldy # 0
  sta (zPtrGen),y
number1:
  dec zMem2
  bpl number0
  rts

;---------------------------------------------------------------------------


.if NEW_FRONTEND		; Board draw routines 

tdrawoffset:		; Draw offset list (rev order)
  .byte VDUX*2+1,VDUX*2, VDUX+1,VDUX,1,0

tpieceoffset:  			; Index to (tpiecechr): 0=grey, 1=empty, 3=wm, 4=wk, 6=bk, 7=bm
  .byte 0,6,6,12,18,6,24,30
  .byte 0,6,6,12,18,6,18,12
  
tpiecechr: 
  .byte 102,102,102,102,102,102	; Grey sq
  ;.byte 58,58,58,58,58,58	; Grey sq
  .byte 32,32,32,32,32,32	; Empty
.if USE_HIRES=0  	; Use normal PETSCII
  .byte 233,223,95,105,32,32  ; White man
  .byte 81,81,95,105,32,32	; White king
  .byte 87,87,74,75,32,32	; Black king
  .byte 85,73,74,75,32,32 	; Black man
.endif

.if USE_HIRES			; Use hires char set
  .byte CUSTOMCHR+00, CUSTOMCHR+03, CUSTOMCHR+01, CUSTOMCHR+04,CUSTOMCHR+02, CUSTOMCHR+05    ; White man
  .byte CUSTOMCHR+06, CUSTOMCHR+09, CUSTOMCHR+07, CUSTOMCHR+10,CUSTOMCHR+08, CUSTOMCHR+11    ; White king
  .byte CUSTOMCHR+18, CUSTOMCHR+21, CUSTOMCHR+19, CUSTOMCHR+22,CUSTOMCHR+20, CUSTOMCHR+23    ; Black king
  .byte CUSTOMCHR+12, CUSTOMCHR+15, CUSTOMCHR+13, CUSTOMCHR+16,CUSTOMCHR+14, CUSTOMCHR+17    ; Black man
.endif

.if 0				; Use (2x2) hires char set
  .byte CUSTOMCHR+00, CUSTOMCHR+02, CUSTOMCHR+01, CUSTOMCHR+03,32,32    ; White man
  .byte CUSTOMCHR+04, CUSTOMCHR+06, CUSTOMCHR+05, CUSTOMCHR+07,32,32    ; White king
  .byte CUSTOMCHR+12, CUSTOMCHR+14, CUSTOMCHR+13, CUSTOMCHR+15,32,32    ; Black king
  .byte CUSTOMCHR+08, CUSTOMCHR+10, CUSTOMCHR+09, CUSTOMCHR+11,32,32    ; Black man
  ;.byte CUSTOMCHR+04, CUSTOMCHR+06, CUSTOMCHR+05, CUSTOMCHR+07,32,32    ; Black king	(COL DIFF)
  ;.byte CUSTOMCHR+00, CUSTOMCHR+02, CUSTOMCHR+01, CUSTOMCHR+03,32,32    ; Black man	(COL DIFF)
.endif 	

draw1piece:		; Draw a piece at (zMem2) (if Carry set, draw gray sqr) (PET:DD=$0e55)
  ldx zMem2
  lda # (2*SQRY)-1	; Count of chrs to do: 5 if (SQRY=3), 3 if (SQRY=2)
  sta zMem7
  lda tboard,x 
  and # $07		
  adc #0		; carry clr if grey, set otherwise.
  			; A is: 0=grey, 1=empty, 3=wm, 4=wk, 6=bk, 7=bm  
  tax

.if USE_COLOR  		; Decide what color to use..
  lda color_black
  cpx #6
  bcs drawpc3  		; bm/bk..
  lda color_white
  cpx #3
  bcs drawpc3  		; wm/wk..
  lda color_ink
drawpc3:  
  sta color_temp		; Color to use later..
.endif  

.if USE_COLOR  
  txa
  clc
  adc graphics_mode		; Graphics_mode= +8 for alt graphics..
  tax
.endif  
  
  lda tpieceoffset,x		; Convert to index to (tpiecechr)
  tax
drawpc2:  
  ldy zMem7
  .if SQRY = 2
    lda tdrawoffset+2,y
  .else
    lda tdrawoffset,y
  .endif
  tay
  lda tpiecechr,x
  sta (zPtrGen),y
.if USE_COLOR  
  lda zPtrGen+1			; Switch ptr to color ram
  eor # VDU_COLOR_EOR
  sta zPtrGen+1
  lda color_temp			; Draw color..		
  .if COLOR_TRANSLATE
    jsr io_color_translate	; Translate color from index to actual colour to use. (C16) (X & Y must be preserved)
  .endif
  sta (zPtrGen),y
  lda zPtrGen+1			; Switch ptr back to VDU
  eor # VDU_COLOR_EOR
  sta zPtrGen+1
.endif
  inx
  dec zMem7
  bpl drawpc2
  rts

draw_calcpos:		; Calc draw pos (zPtrGen) for piece (x) on vdu, ret carry clear if grey square (Generic version)
  stx zMem2
  lda # >pVDU		; hi byte of vdu ptr
  sta zPtrGen+1
  txa 
  and #$07
  asl 
  sta zPtrGen		; store (xpos*2) in low byte ptr.
  txa
  lsr 
  lsr 
  lsr 
  sta zMem1		; save y pos (0-7)
  tay 			; y=vertical screen pos (0-7)
  beq drawd4
drawd6:			; Use repeat addition to calc VDU ypos..
  clc 
  lda # VDUX*SQRY		; Advance SQRY lines
  adc zPtrGen
  sta zPtrGen
  bcc drawd5
  inc zPtrGen+1
drawd5:
  dey 
  bne drawd6
drawd4:			; Now see if its "grey" sqr
  txa 			; get x pos (0-7)
  clc 
  adc zMem1		; Add in ypos (0-7)
  lsr			; carry  clr if grey sqr..
  rts
   
  
drawpieces:		; Draw all pieces.
  ldx #$3f		; Draw pieces on VDU (4x3)
drawpc1:
  jsr draw_calcpos
  jsr draw1piece
  ldx zMem2
  dex
  bpl drawpc1
  rts
  
.endif 			; (NEW_FRONTEND)

draw_board_only:
  jsr reflectbrd		; (historic kludge - reflect board before draw)
  jsr drawpieces
  jmp reflectbrd

draw_board:			; Draw full board with border, side text etc (PET:BD=$098a, page i7) (OLD VERSION) 
  jsr draw_board_only		; Draw pieces only..
  .if (NEW_FRONTEND = 2)
  .else
    jsr draw_borders  		; Draw lines around board
    jsr draw_sidetext  		; Draw assorted text info & graphics on right..
  .endif
				; Drop thro to drawiq/drawclock..
drawiq:
  lda iqcut			; Page i8
  lsr 
  jsr convbyte2dec	; display IQ 
  stx pVDU_IQ		;   (pVDU+$205)
  sta pVDU_IQ+1

drawclock:   				; Display clock on vdu (PET:BT=$0a08)
.if USE_CLOCK
  ldy #$05
  ldx #$04
drawbb:
  cpy #$03
  bne drawba
  dey
drawba:
  lda clockwhite,x
  sta pVDU_CLOCKWH,y	; VDU white clock (pVDU+$111)
  lda clockblack,x
  sta pVDU_CLOCKBL,y	; VDU black clock (pVDU+$189)
  dey
  dex
  bpl drawbb
.endif			; USE_CLOCK
  rts

.if (NEW_FRONTEND = 0) + (NEW_FRONTEND = 1)		; Draw assorted text info & graphics on right..

draw_sidetext:
  ldx #$07
drawb9:
.if USE_CLOCK
  lda #$a0
  sta pVDU_CLOCKTOP+VDUX*2,x 	;   (pVDU+$110)
  sta pVDU_CLOCKTOP+VDUX*3,x 	;   (pVDU+$138) 
  sta pVDU_CLOCKTOP+VDUX*5,x	;   (pVDU+$188)
  sta pVDU_CLOCKTOP+VDUX*6,x	;   (pVDU+$1b0)
  lda tmsgvduclock,x  		; print "CLOCK"
  sta pVDU_CLOCKTOP,x 		;   (pVDU+$c0)
  lda tmsgvduwhite,x		; print "WHITE" 
  sta pVDU_CLOCKTOP+VDUX,x	;   (pVDU+$e8)
  lda tmsgvdublack,x  		; print "BLACK"
  sta pVDU_CLOCKTOP+VDUX*4,x	;   (pVDU+$160)
  lda # CHR_HLINE
  sta pVDU_INFO,x 		;   (pVDU+$20)
  ;  sta pVDU_INFO+VDUX*16,x 	;   (pVDU+$2a0)
.endif
			; Draw side texts etc. Version Hardwired for 40 columns..
  cpx #$04
  bcs drawb8
  lda tmsgvduiq,x 
  sta pVDU_IQ-3,x 		; print "IQ="
  lda tmsgvdu,x 
  sta pVDU_INFO+VDUX+2,x 	; print "ORAC"
  lda tmsgvdu+4,x 		; print "5.xx"
  sta pVDU_INFO+VDUX*2+2,x 	;   (pVDU+$72)
drawb8:
  dex 
  bpl drawb9
  lda #$3a			; colon ":"
  sta pVDU_MOVE			;   (pVDU+$2f1)
.if USE_CLOCK  			; Draw clock surround
  sta pVDU_CLOCKWH+3	 	;   (pVDU+$114)
  sta pVDU_CLOCKBL+3		;   (pVDU+$18c)
  lda # CHR_TOPLEFT		; Top left corner shape
  sta pVDU_INFO			;   (pVDU+$20)
  ;  sta pVDU_INFO+VDUX*16		;   (pVDU+$2a0)
  lda # CHR_TOPRIGHT		; Top right corner shape
  sta pVDU_INFO+7		;   (pVDU+$27)
  lda #CHR_LINERIGHT
  sta pVDU_INFO+VDUX+7		;   (pVDU+$4f)
  sta pVDU_INFO+VDUX*2+7	;   (pVDU+$77)
  sta pVDU_INFO+VDUX*3+7	;   (pVDU+$9f)
.endif 			; USE_CLOCK  
  rts

draw_borders:  		; Draw lines around board
  lda #$63		; Draw line under board
  ldx # SQRX*8-1
draw_bor2:
  sta pVDU + VDUX*SQRY*8,x 
  dex 
  bpl draw_bor2
			; Draw line down right of board
  ldx # SQRY*8
  lda #<(pVDU_INFO)
  sta zPtrGen
  lda #>(pVDU_INFO)  
  sta zPtrGen+1
  ldy #$00			
draw_bor3:
  lda #$65
  sta (zPtrGen),y
  tya
  clc
  adc #VDUX
  tay
  bcc draw_bor4
  inc zPtrGen+1
draw_bor4:  
  dex
  bne draw_bor3
  rts

.endif

;---------------------------------------------------------------------------

evalparms:		; Calc Simple Material count, scan brd. (valwhite,valblack,nkings) (VW,VB,VT,NK) (PET:E0=$0a22) (man=2. king=3)
  lda #$00
  sta valwhite
  sta valblack
  sta nkings
  ldx #$3f
eval4:
  lda tboard,x
  bpl eval1		; White piece (or empty)
  eor #$ff		; Add black piece (2,3) to valblack
  sec 
  adc valblack
  sta valblack
  bne eval2
eval1:			; Add white piece (2,3) to valwhite
  clc 
  adc valwhite
  sta valwhite
eval2:
  lda tboard,x 
  lsr 
  bcc eval3
  inc nkings
eval3:
  dex 
  bpl eval4
  lda #$74		; Init time value. This has sq vals added as search goes up tree.
  sta ctime
  rts

;---------------------------------------------------------------------------


.if USE_CLOCK

interrupt:   		; IRQ replacement code, update clocks etc (ignoring ABORT)  (PET:IS=$0a8a)
  ldx #$05		; White clock, index last digit.
  lda side2move		; Clock update, which colour ?
  beq interix
  ldx #$0b		; Black clock, index last digit.
interix:		; Increment clock..
  inc clockwhite,x
  lda clockwhite,x
  cmp #$6c		; (asc("0") + 60) Count 60 ticks for 1 second.
  bcc interiy		; No change to clock..
interiz:
  lda #$30		; Wrap back to "0"
  sta clockwhite,x
  dex
  bmi interiy
  inc clockwhite,x
  lda clockwhite,x
  cmp clock_maxdigit,x
  bcs interiz
  jsr drawclock			; Display clock on vdu
interiy:
  .if USE_RANDMODE	; Randomise low bits of eval with clock
    lda zp_jiffy_low
    ; lda $ROM_RAND,x	; Use a page of ROM for randomisation ($fc00)
    and rand_mode	; mask off bits. (ie rand_mode=1 just randomises lowest bit of eval)
    sta rand_val	; Save, EOR with eval (zEvalTot) during search..
  .endif
  
  jmp romirq			; Continue normal ROM IRQ (C64:$ea31) was: jmp (voldirq) 

.endif 		; USE_CLOCK

;---------------------------------------------------------------------------

moveonscr:		; Display a move (bestmove,bestdir) on the screen (PET:M0=$0ae3, page i11)
  .if USE_SOUND
     jsr io_sound		; Make a beep sound (via machine specific routine)
  .endif
  ldx bestmove
  stx zMem3
  lda #$00
  sta zMem5
  lda tboard,x
  sta zMem4
  jsr movem1
  lda bestmove
movem3:
  clc
  adc bestdir
  tax
  lda tboard,x
  beq movem2
  stx zMem3
  lda #$00
  jsr moveon1
  lda zMem3
  clc
  bcc movem3
movem2:
  ldy zMem4
  sta zMem4
  stx zMem3
  tya
  lsr
  tya
  bcs movem9

movem0:
  cpx #$08
  bcc movem7
  cpx #$38
  bcc movem9
movem7:

  cmp #$80
  lda #$03
  ldy #$00
  sty mvoption

  bcc movem9
  lda #$fd
movem9:
  sta zMem5
  jsr movem1

.if (USE_SOUND=2)
   jsr io_sound_off		; End sound/beep.
.endif

  rts

movem1:				; (PET:M1=$0b3d)
  lda #$03
  sta zMem6
movem5:
  lda zMem4
  jsr movem4
  lda zMem5
  jsr movem4
  dec zMem6
  bne movem5
  rts

moveon1:
  ldy #$80
  sty mvoption
movem4:				; (PET:M4=$0b50)
  ldx zMem3
  sta tboard,x 
  ; ldy #$02			; len of move flash
movem6:
  jsr io_wait_jiffy2		; wait for 2 vdu vertical retrace (1/60 sec)
  ; dey 
  ; bne movem6
  txa 
  pha 
  jsr reflectbrd
  pla 
  eor #$07
  tax 
  jsr draw_calcpos
  jsr draw1piece
  jmp reflectbrd
   
;---------------------------------------------------------------------------

humancmd:   		; Input and execute human command   (PET:H0=$0b70, page i13)
  jsr showcolour
  jsr inputcmd
  ldy #$00
  jsr getuserin
  bcc humanmove        	; Numeric char, move NN-NN.:
  ldx #$13
humanh3:   		; See if CMD letter.:
  cmp cmdchars,x
  beq humanh2		; Yes, execute.:
  dex
  bpl humanh3
humanerror:
  jsr displaywhat
  clc
  bcc humancmd
humanh2:   		; Execute CMD pointed to by X
  iny
  jsr humanh4		; Push CMD address on stack
  jsr draw_board
  clc
  bcc humancmd		

humanh4:			; Push CMD (x) address on stack & call..
  lda cmdaddrhi,x
  pha
  lda cmdaddrlo,x
  pha
humanret:
  rts

humanmove:   			; Human input numeric, so check move NN-NN  (PET:H1=$0ba1, page i14)
  jsr convdec2by	; Read int from cmd line
  jsr conva2coord	; Conv to BRD co-ord
  sta zMem1
  jsr getuserin
  cmp #$2d			; '-' in middle ?
  bne humanerror	; No, so disp 'WHAT'
  iny
  jsr convdec2by	; Read 2nd int from cmd line
  bcs humanerror
  jsr conva2coord	; Conv to BRD coord
  pha				; Stash BRD dest
  lda zMem1
  pha				; Stash BRD source
  jsr prepcompmove
  jsr genlegalmoves	; Generate list of legal moves
  pla
  sta bestmove
  pla
  sec
  sbc bestmove
  cmp #$0a
  bcc humanh7
  cmp #$f7
  bcs humanh7
  cmp #$80
  ror
humanh7:
  sta bestdir
  ldy nmoves		; Ok, see if entered human move is in leg MOVE tab
humanh6:
  dey
  bmi humanerror	; No it isnt, so illegal move.:
  lda (zPtrMov),y
  cmp bestmove
  bne humanh6
  lda (zPtrDir),y
  cmp bestdir		; Patch here for JSAVEUNDO
  bne humanh6
			; Yes, matching move found, so legal move
dobestmove: 		; Execute move BESTMOVE, BESTDIR on scr & brd   (PET:HO=$0bee, page i14)
  lda #$00
  sta mvoption
  jsr moveonscr
  lda zMem3
  ldy mvoption

  beq dobestret
  sta zmove
  jsr genforsqr
  lda #$00
  ldx mvgenflag
  beq dobesth8
  lda zmove
  ora #$80		; Return minus flag
dobesth8:
  sta mvoption
dobestret:
  rts


		; Display a message for user..

.define DISP_MSG_LEN 7		; 7 chars long

displaywhat:
  ldx # (tmsgwhat-tmsgvdu)	; print "WHAT" (was offset $2d)

displaymsg:			; Display message at (tmsgvdu+X) (PET:H5=$0c07)
  ldy # (256-DISP_MSG_LEN)	
displaymsgh9:
  lda tmsgvdu,x
  sta pVDU_MSG -(256-DISP_MSG_LEN),y	; VDU msg display area (pVDU+$3b9)
  inx 
  iny
  bne displaymsgh9
  rts

;---------------------------------------------------------------------------
   
setiq:				; CMD:i##  Set iq level (PET:HI=$0c16)
  jsr convdec2by
  cmp #$28
  bcs displaywhat
  asl 
  sta iqcut
  rts
   
passcmd:			; CMD: P (unused command) (PET:HP=$0c22)
  pla				; pop stack and return
  pla 
  rts

;---------------------------------------------------------------------------
   
setwhite:			; CMD:W##  Set white piece (PET:HW=$0c25)
  ldx # pwhiteman
  .byte $2c			; Skip instruction - classic 6502 BIT trick..
setblack:			; CMD:B##  Set black piece (PET:HB=$0c28)
  ldx # pblackman
  jsr getuserin
  bcc setpiecehc
  cmp #$0b			; char K for King? (BK# or WK#)
  bne displaywhat
  iny 
  inx 
  bpl setpiecehc
  dex 
  dex 
  .byte $2c			; Skip instruction - classic 6502 BIT trick..
erasepiece:			; CMD:E##  Erase piece (PET:HE=$0c3a)
  ldx #$00
setpiecehc:
  stx zMem1			; Piece type..
  jsr convdec2by
  bcs displaywhat
  beq displaywhat
  cmp #$21
  bcs displaywhat
  jsr conva2coord
  tax 
  lda zMem1
  sta tboard,x 
  rts

conva2coord:
  sta zPtrGen+1
  jmp conv2coord

;---------------------------------------------------------------------------

			; (See I15/O0, $0c58)
gencompmove:   		; Find ORACs BESTMOVE (=$ff lost), BESTDIR:   (PET:O0=$0c58, page i15)
  jsr showcolour	; Print W/B on scr, set CURCOLOR
  lda #$69
  sta pVDU_SIDE+1	; 'Thinking' square (pVDU+$36a)
  ldx #$00
  dex
  stx tbestmove         ; $FF means no moves
  lda mvoption
  pha                   ; Stash
  lda iqcut
  pha
  lda #$0c              ; Quick look for win/loss
  sta iqcut
  jsr prepcompmove
  jsr oracbrain
  pla
  sta iqcut
  pla
  sta mvoption
  ldx tmoveevallo
  jsr patchcarryifwinloss
  bcc gencompo1		; Loss.:
  ldx tnomoves
  dex
  beq gencompo1		; No moves, loss.:
  ldx #$ff
  stx tbestmove		; $FF means none found.:
  jsr prepcompmove
  jsr oracbrain         ; The real thing.:
gencompo1:
  ldx tbestmove
  stx bestmove
  lda tbestdir
  sta bestdir
  rts

showcolour:			; Show "W"/"B" current colour char on vdu  (PET:OC=$0cad)
  lda #$20
  sta pVDU_SIDE+1 		; (pVDU+$36a)
  ldx #$17			; "W"..
  lda side2move
  sta curcolor
  bpl showcod
  ldx #$02			; "B"..
showcod:
  stx pVDU_SIDE
  rts

prepcompmove:			; Prepare for computer move, init stack ptrs etc  (PET:OM=$0cc2)
  lda #$00
  sta cply
.if SHOW_MOVE_INFO		; Increment node count..
  sta nodecount;
  sta nodecount+1;
  sta nodecount+2;
.endif  
  sta interest
  lda # <tmovestack		; Init ptr to (tmovestack) (PET:$1336)
  sta zPtrMov
  lda # >tmovestack
  sta zPtrMov+1
  lda # <tdirstack		; Init ptr to (tdirstack) (PET:$148a)
  sta zPtrDir
  lda # >tdirstack
  sta zPtrDir+1
  jsr evalparms
  lda #$01
  sta lowerply
  lda # MAX_PLY-3			; Max ply depth (PET:$1b)
  sta upperply
  rts

;---------------------------------------------------------------------------

execcompmove:   	; Find and execute computer move (Carry set = resigns)   (PET:OE=$0cea, page i17)
  jsr gencompmove	; Generate move BESTMOVE, BESTDIR
  cpx #$ff
  beq execcompox	; Unable to win, so carry set to resign:
  stx zPtrGen			; Convert BESTMOVE to ascii BRD #
  jsr coord2brd
  jsr convbyte2dec
  stx pVDU_MOVE+1 	; and put on screen (pVDU+$2f2)
  sta pVDU_MOVE+2
  lda #$2d
  sta pVDU_MOVE+3	; '-' on screen
  jsr dobestmove	; Execute move BESTMOVE, BESTDIR on scr & brd
  lda zMem3			; Convert dest to ascii brd no:
  sta zPtrGen
  jsr coord2brd
  jsr convbyte2dec
  stx pVDU_MOVE+4		; And print on screen:
  sta pVDU_MOVE+5
  clc			; Indicates Not resigning
execcompox:
  rts

;-----------------------------------------------------------------------------------

.if SMALLER_CODE = 0

tmsghelp:	; ORAC help text, written with routine (textout)

.if IS_V20	;  VIC20 22 column version...

 .byte VDUX*3-1,   "  ORAC DRAUGHTS ",VERSION1,".",VERSION2,VERSION3,0
 .byte VDUX*2,   "(C)A.MILLETT 1980-2024"		,0
 .byte VDUX,     "    GPL3/FREEWARE.",0
 .byte VDUX*2,   "#-# <RET> ENTER MOVE"		,0 
 .byte VDUX,     "OR USE CURSOR+SPACE"	,0
 .byte VDUX,     "N <RET> SHOW NUMBERS"		,0
 .byte VDUX,     "I# <RET> SET IQ LEVEL"	,0
 .byte VDUX,     "A <RET> AUTO PLAY"		,0
 .byte VDUX,     "G <RET> COMP PLAYS"		,0
 .byte VDUX,     "X <RET> SWAP SIDE"		,0
 .byte VDUX,     "C <RET> CLR BOARD"		,0
 .byte VDUX,     "R <RET> RESTART"		,0
 .byte VDUX,     "B#,BK#,W#,WK# SET POS"	,0
 .byte VDUX,     "L#,S# LOAD/SAVE POS"		,0
 .byte VDUX,     "F1/F3/F5/F7 COLOURS"		,0
 .byte VDUX*2,   " HIT SPACE TO PLAY"			,0,0

.else			; 40 column version

 .byte VDUX*3+5, "--- ORAC DRAUGHTS ",VERSION1,".",VERSION2,VERSION3," ---",0
 .byte VDUX*2-6, "(C) A.MILLETT 1980-2024. GPL3/FREEWARE.",0
 .byte VDUX*2,   "#-# <RET> ENTER MOVE, IE 11-15."	,0 
 .byte VDUX,     "  OR CURSOR + SPACE SELECTS MOVE."		,0
 .byte VDUX,     "N <RET> SHOW NUMBERS."		,0
 .byte VDUX,     "I# <RET> SET IQ LEVEL, IE: I22."	,0
 .byte VDUX,     "A <RET> COMP AUTO PLAY."		,0
 .byte VDUX,     "G <RET> COMP PLAYS YOUR POS."		,0
 .byte VDUX,     "X <RET> SWAP SIDE TO MOVE."		,0
 .byte VDUX,     "C <RET> CLEAR BOARD."			,0
 .byte VDUX,     "R <RET> RESTART GAME."		,0
 .byte VDUX,     "B#,BK#,W#,WK# <RET> SET POS, IE B15."	,0
 .byte VDUX,     "L#,S# <RET> LOAD/SAVE POS, IE:L4."	,0
 .if IS_C16
  .byte VDUX,     "CTRL-D/E/F/G - SET COLOURS."		,0
 .else
  .byte VDUX,     "F1/F3/F5/F7 - SET COLOURS."		,0
 .endif
 .byte VDUX*3+10,"HIT SPACE TO PLAY"			,0,0

.endif		; IS_V20

zpVdu = zMem1

textout:		; Output text at (X;A) to screen. input format: .byte SKIP,"Text",0, .. Double 0,0 ends
  sta zPtrGen
  stx zPtrGen+1
  lda # <(pVDU)
  ldx # >(pVDU)
  sta zpVdu
  stx zpVdu+1
text_next:  
  ldy #0
  lda (zPtrGen),y
  beq textout_done
  iny
  clc
  adc zpVdu
  sta zpVdu
  bcc textout1
  inc zpVdu+1
textout1:
  lda (zPtrGen),y
  beq textout2
  and #63		; Conv ascii to petscii (simple)
  sta (zpVdu),y
  iny
  bne textout1
textout2:
  tya 
  sec
  adc zPtrGen
  sta zPtrGen
  bcc text_next
  inc zPtrGen+1
  bne text_next
textout_done:
  rts

;-----------------------------------------------------------------------------------

help_screen:   		; Old Title screen sub   (PET:F0=$0d19, page i18)
  .if USE_CLOCK
    jsr io_clockoff		; Turn clock off
  .endif 
  .if USE_COLOR
    jsr io_color_set		; Set ink/background colours..
  .endif
  jsr clearscr
  
  ldx # VDUX-1
  lda # 102
frontf2:
  sta pVDU,x
  sta pVDU+(VDUX*(VDUY-1)),x
  dex
  bpl frontf2

.if 0		; Alt border..

  ldx # VDUX-1		; Display line of hires pieces top/bottom of screen
frontf2:
  ;lda # 102
  ;sta pVDU+(VDUX*(VDUY-1)),x
  txa
  and #1
  beq frontf3
  lda #3
frontf3:  
  clc
  adc #CUSTOMCHR	; (was 199 on C64)
  sta pVDU,x
  adc # 1
  sta pVDU+VDUX,x
  adc # 1
  sta pVDU+VDUX*2,x
  dex
  bpl frontf2
  
.endif	
  
  lda # <tmsghelp		; Display help text..
  ldx # >tmsghelp
  jsr textout

frontf4:
  ; lda shiftkeyflag	  		; Wait for shift key..
  ; beq frontf4
  jsr kerngetin			; Read a key from keybuf ($ffe4)
  cmp #$20
  bne frontf4			; Wait for space key..
  ;  jmp clearscr    		; Drop through to cls..

.else			; (SMALLER_CODE)
  
help_screen:   			; Drop through to cls..

.endif			; (SMALLER_CODE)

clearscr:		; Clear screen  (PET:FC=$0d62)
  lda #$20
  ldx #$00
clearsc1:
  sta pVDU,x
  sta pVDU+$100,x
  .if (VDUY=25) + (V20_RELOC) 		; For 40x25 computers..
    sta pVDU+$200,x
    sta pVDU+$300,x
  .endif
  dex
  bne clearsc1
  rts

;------------------------------------------------------------------------------

newgame:		; Start a new game..
  
.if SMALLER_CODE 
  jsr clearscr
.else
  jsr help_screen
.endif

.if USE_CLOCK
  sei			; Enable clock IRQ..
  lda #<interrupt
  sta vectirq
  lda #>interrupt
  sta vectirq+1
  
  ldx #$0b		; Clear clock digits, set all to "0"..
  lda #$30
init1:
  sta clockwhite,x
  dex
  bpl init1
.endif		; USE_CLOCK
  cli

  lda #$24			; Default iq=18 
  sta iqcut
  
  lda #$80
  sta side2move			; Black to move..
  jsr setupnewbrd		; New board
  jsr draw_board

mainloop:			; Main program loop (PET:L2=$0da7, page i19)
  jsr showcolour		; Display W/B side to move...
  ;brk
  jsr prepcompmove
  jsr genlegalmoves		; Generate legal moves..
  ldx # (tmsgiwin - tmsgvdu)	; offset for "I WIN"  (was $46)
  lda nmoves	
  beq maindohumanl1		; If no moves, then human loses..

  ldx # (tmsgjump - tmsgvdu)	; offset for "JUMP".. (was $34)
  lda takeflag
  bne maindohumanl1
  
  ldx # (tmsgmove - tmsgvdu)	; offset for "MOVE"..  (was $3b)
maindohumanl1:
  jsr displaymsg		; print "JUMP" or "MOVE " or "I WIN"..

  ; jsr patchauto0f8f
  ldx pVDU_INPUT		; Is auto mode on ?
  cpx #1			; is 'A'?
  bne patchauto1
  lda key_buffer_size
  beq mainloopcomp		; Skip human input, tell comp to take another move..
patchauto1:
  
  jsr humancmd

  lda mvoption

  bmi mainloop
  
  jsr exchange_side		; swap side to move W<->B
  ; lda side2move			; change side to move..
  ; eor #$80
  ; sta side2move

mainloopcomp:   		; Comp Go (PET:L4=$0dd5,page i20)
  jsr execcompmove
  bcc mnext	
  ldx # (tmsgyouwin - tmsgvdu)		; print "YOU WIN"  (was $4d)
;  jsr displaymsg
  jmp maindohumanl1			; old code stopped on loss, now just cont..
mnext:

  lda mvoption

  bmi mainloopcomp
  lda side2move
  eor #$80
  sta side2move
  jmp mainloop

;------------------------------------------------------------------------------

.if NEW_FRONTEND=0	; Original (old) board/piece drawing routines..

drawpieces:		; Draw pieces on screen (PET:D0=$0df2)
  ldx #(tmsgclear - tmsgvdu)	; Clear msg window (was $3f)
  jsr displaymsg
  
  ldx #$3f		; Draw pieces on VDU (4x3)
drawd1:
  jsr draw_calcpos
  jsr draw1piece
  dec zMem2
  ldx zMem2
  bpl drawd1
  rts
  
  
draw_calcpos:		; Calc draw pos (zPtrGen) for piece (x) on vdu, ret carry clear if grey square (PET:DP=$0e2b)
  stx zMem2
  lda # >pVDU		; hi byte of vdu ptr
  sta zPtrGen+1
  txa 
  and #$07
  asl 
  asl 
  sta zPtrGen		; store (xpos*4) in low byte ptr.
  txa
  lsr 
  lsr 
  lsr 
  sta zMem1
  tay 			; y=vertical screen pos (0-7)
  beq drawd4
drawd6:
  clc 
  lda # VDUX*3		; Advance 3 lines ($78)
  adc zPtrGen
  sta zPtrGen
  bcc drawd5
  inc zPtrGen+1
drawd5:
  dey 
  bne drawd6
  
drawd4:			; Now see if its "grey" sqr
  txa 			; get x pos (0-7)
  clc 
  adc zMem1		; Add in ypos (0-7)
  lsr			; lo bit clr if grey sqr..
  rts

draw1piece:		; Draw a piece at (zMem2) (if Carry clr, draw gray sqr) (PET:DD=$0e55)
.if USE_COLOR  		; Decide what color to use..
  ldx color_ink
  stx color_temp		; Color to use later..
.endif  
  bcc drawshape_grey
  ldx #$00
  jsr drawshape		; Draw Empty square (fill with $20)
  ldx zMem2
  lda tboard,x 

.if USE_COLOR  		; Decide what color to use..
  beq drawpc5
  ldx color_black
  cmp #128
  bcs drawpc4  		; bm/bk..
  ldx color_white
drawpc4:
  stx color_temp		; Color to use later..
drawpc5:
.endif  

  and #7		; Convert to index: wm=2, wk=3, bk=5, bm=6

.if USE_COLOR
  clc
  adc graphics_mode		; if graphics_mode set, just use colored wht man/king gfx
.endif

  tax
  lda tpiece2index,x 
  tax
  
.if 0		; OLD code
  ldx #$03
drawd8:
  cmp tpiece2shape,x 	; Search piece table..
  beq drawd9
  dex 
  bpl drawd8
  bmi drawda		; Not found, empty square..
drawd9:
  lda tpiece2index,x 
  tax			; Index into shape table..
.endif

  jsr drawshape
drawda:
  rts
   
drawshape_grey:
  ldx #$19		; Index for grey sqr shape..
drawshape:		; Draw a char (x) shape from table (tpieceshapes) (chr,offset,chr,offset..NULL) (PET:P0=$0e7b)
  lda tpieceshapes,x 
  beq drawshap1
  inx 
  ldy tpieceshapes,x 
  sta (zPtrGen),y 
.if USE_COLOR  
  lda zPtrGen+1			; Switch ptr to color ram
  eor # VDU_COLOR_EOR
  sta zPtrGen+1
  lda color_temp			; Draw color..		
  .if COLOR_TRANSLATE
    jsr io_color_translate	; Translate color from index to actual colour to use. (C16) (X & Y must be preserved)
  .endif
  sta (zPtrGen),y
  lda zPtrGen+1			; Switch ptr back to VDU
  eor # VDU_COLOR_EOR
  sta zPtrGen+1
.endif
  inx 
  bne drawshape
drawshap1:
  rts
   
.endif		; (NEW_FRONTEND=0)

.if USE_CLOCK
clock_maxdigit:			; Clock - Highest digits for each position (PET:CL=$0e8a)
 .byte $ff,$36,$3a,$36,$3a,$ff,$ff,$36,$3a,$36,$3a,$ff
tmsgvduclock:
.byte $a0,$83,$8c,$8f,$83,$8b,$a0	; <RVS>" CLOCK " 
tmsgvduwhite:
.byte $a0,$97,$88,$89,$94,$85,$a0	; <RVS>" WHITE "
tmsgvdublack:
.byte $a0,$82,$8c,$81,$83,$8b,$a0,$a0	; <RVS>" BLACK "
.endif 		; USE_CLOCK

tmsgvdu:			; Assort screen texts (PET:TA=$0e96)
  .byte $0f,$12,$01,$03		; "ORAC"
  .byte VERSION1,$2e,VERSION2,VERSION3	; "5.XX"  version no

tmsgvduiq:
 .byte $09,$11,$3d,$20					; "IQ="
 ; .byte $01,$02,$0f,$12,$14,$20,$3f			; "ABORT ?"
tmsgwhat:
 .byte $17,$08,$01,$14,$20,$3f,$20			; "WHAT ? " 
tmsgjump:
 .byte $0a,$15,$0d,$10,$20,$21,$20			; "JUMP ! "  
tmsgmove:
 .byte $0d,$0f,$16,$05,$20				; "MOVE   " 
tmsgclear:						
 .byte $20,$20,$20,$20,$20,$20				; "       "
tmsgiwin:
 .byte $20,$09,$20,$17,$09,$0e,$20			; "I WIN  "
tmsgyouwin:
 .byte $19,$0f,$15,$20,$17,$09,$0e			; "YOU WIN"
 
; .byte $04,$12,$01,$17,$0e,$20,$21			; "DRAWN !"



cmdchars:		; CMD CHARS: "INXPARCVSLBEW+-=^GA "   (PET:T0=$0f52) len=20
     ; I   N   X   P   H   R   C   V   S   L   B   E   W   M   -   =   F   G   A  spc 
.byte $09,$0e,$18,$10,$08,$12,$03,$16,$13,$0c,$02,$05,$17,$0d,$2d,$3d,$06,$07,$01,$20

	; Return addresses-1 for all 20 cmds, 20 low bytes, then 20 hi bytes
cmdaddrlo:				; (PET:T1=$0f66, len 20)

.byte <(setiq-1), <(patchnumberbrd-1), <(exchange_side-1), <(passcmd-1), <(help_screen-1), <(newgame-1), <(erasebrd-1), <(change_graphics-1)
.byte <(save_board_cmd-1), <(load_board_cmd-1), <(setblack-1), <(erasepiece-1), <(setwhite-1),  <(set_rand_mode-1),  <(draw_board-1)
.byte <(draw_board-1),  <(flipbrd-1), <(patchcmdgo-1), <(patchcmdgo-1), <(humanret-1 )

; orig .byte $15,$9f,$1c,$21,$89,$75,$11,$89,$88,$7f,$27,$39,$24,$89,$89,$89,$89,$98,$98,$9f

cmdaddrhi:				; (PET:T1+20=$0f7a, len 20)

.byte >(setiq-1), >(patchnumberbrd-1), >(exchange_side-1), >(passcmd-1), >(help_screen-1), >(newgame-1), >(erasebrd-1), >(change_graphics-1)
.byte >(save_board_cmd-1), >(load_board_cmd-1), >(setblack-1), >(erasepiece-1), >(setwhite-1),  >(set_rand_mode-1),  >(draw_board-1)
.byte >(draw_board-1),  >(flipbrd-1), >(patchcmdgo-1), >(patchcmdgo-1), >(humanret-1)

; orig .byte $0c,$06,$09,$0c,$09,$0d,$09,$09,$06,$06,$0c,$0c,$0c,$09,$09,$09,$09,$0f,$0f,$0b


.byte $00

;patchauto0f8f:   		; Called by MAIN, check for @ mode (PET:$0f8f)
;  ldx pVDU_INPUT
;  cpx #1		; is 'A'?
;  bne patchauto1
;  lda key_buffer_size
;  beq patchgo1
;patchauto1:
;  rts

patchcmdgo:   		; CMD:G Called by @,G cmds:  (PET $0f99)
  pla
  pla
patchgo1:
  pla
  pla
  jmp mainloopcomp

patchcarryifwinloss:
  cpx tmoveevalhi
  beq patchcarry1
  sec
  rts
   
patchcarry1:
  inx 
  cpx #$02
  rts

.if NEW_FRONTEND=0	; Original (old) board/piece drawing tables..

tpiece2index:			; ..indexs into (tpieceshapes)..
  .byte 0,0,$3f,$32,0,$59,$4c,0
  .byte 0,0,$3f,$32,0,$32,$3f,0
;tpiece2shape:			; List of pieces with index into (tpieceshapes) (PET:TD=$1014)
;.byte $03,$02,$fe,$fd
;tpiece2index:			; ..indexs into (tpieceshapes)..
;.byte $32,$3f,$4c,$59

tpieceshapes:             ; Piece shape lists for board pieces (char,offset,char,offset...NULL) 

.if USE_HIRES=0			; orig ORAC PETSCII piece codes. 
 .byte $20,$00,$20,$01,$20,$02,$20,$03,$20,$28,$20,$29,$20,$2a,$20,$2b,$20,$50,$20,$51,$20,$52,$20,$53,$00	; Empty sqr
 .byte $66,$00,$66,$01,$66,$02,$66,$03,$66,$28,$66,$29,$66,$2a,$66,$2b,$66,$50,$66,$51,$66,$52,$66,$53,$00	; Grey sqr
 .byte $e9,$01,$df,$02,$5f,$29,$69,$2a,$e2,$51,$e2,$52,$00							; White king
 .byte $20,$01,$20,$02,$e9,$29,$df,$2a,$e2,$51,$e2,$52,$00							; White man
 .byte $20,$01,$20,$02,$4e,$29,$4d,$2a,$e2,$51,$e2,$52,$00							; Black man
 .byte $4e,$01,$4d,$02,$4d,$29,$4e,$2a,$e2,$51,$e2,$52,$00							; Black king
.endif 

.if USE_HIRES			;  UDG hires piece lists  Codes 193-208 ($c1-$d0)
  .byte $20,$00,$20,$01,$20,$02,$20,$03,$20,$28,$20,$29,$20,$2a,$20,$2b,$20,$50,$20,$51,$20,$52,$20,$53,$00	; Empty sqr
  .byte $66,$00,$66,$01,$66,$02,$66,$03,$66,$28,$66,$29,$66,$2a,$66,$2b,$66,$50,$66,$51,$66,$52,$66,$53,$00	; Grey sqr

  .byte CUSTOMCHR+06,$01,CUSTOMCHR+09,$02,CUSTOMCHR+07,$29,CUSTOMCHR+10,$2a,CUSTOMCHR+08,$51,CUSTOMCHR+11,$52,0	; White king
  .byte CUSTOMCHR+00,$01,CUSTOMCHR+03,$02,CUSTOMCHR+01,$29,CUSTOMCHR+04,$2a,CUSTOMCHR+02,$51,CUSTOMCHR+05,$52,0	; White man
  .byte CUSTOMCHR+12,$01,CUSTOMCHR+15,$02,CUSTOMCHR+13,$29,CUSTOMCHR+16,$2a,CUSTOMCHR+14,$51,CUSTOMCHR+17,$52,0	; Black man
  .byte CUSTOMCHR+18,$01,CUSTOMCHR+21,$02,CUSTOMCHR+19,$29,CUSTOMCHR+22,$2a,CUSTOMCHR+20,$51,CUSTOMCHR+23,$52,0	; Black king
.if 0  
  .byte $20,$01,$20,$02,CUSTOMCHR+4,$29,CUSTOMCHR+6,$2a,CUSTOMCHR+5,$51,CUSTOMCHR+7,$52,0			; White king
  .byte $20,$01,$20,$02,CUSTOMCHR+0,$29,CUSTOMCHR+2,$2a,CUSTOMCHR+1,$51,CUSTOMCHR+3,$52,$00			; White man
  .byte $20,$01,$20,$02,CUSTOMCHR+8,$29,CUSTOMCHR+10,$2a,CUSTOMCHR+9,$51,CUSTOMCHR+11,$52,$00			; Black man
  .byte $20,$01,$20,$02,CUSTOMCHR+12,$29,CUSTOMCHR+14,$2a,CUSTOMCHR+13,$51,CUSTOMCHR+15,$52,0			; Black king
.endif

.endif		; USE_HIRES

.endif 	; (NEW_FRONTEND=0)


tmvorder:               ; Convert 0..31 sqr index into true 0..63 board index, ordered for goodish move order.. (PET:MM=$1082)
.byte $00,$02,$04,$06,$39,$3b,$3d,$3f
.byte $09,$0b,$0d,$12,$14,$16,$19,$1b,$1d,$22,$24,$26,$29,$2b,$2d
.byte $32,$34,$36,$10,$20,$30,$2f,$1f,$0f

; Front end I/O ends.

;-----------------------------------------------------------------------------------
; ORAC checkers playing engine code starts here..
;   More or less original code from 1981!
;-----------------------------------------------------------------------------------

	; piece vals on board
;pwhiteman  = $02
;pwhiteking = $03
;pblackman  = $fe
;pblackking = $fd

pwhite_xor_black = $fc	; unique val when white man xor'd with black man.

evalbridge = $30	; Value of bridge defence at back (sqrs:1,3)
evaldoghole = $08	; Value of dog-hole (ie. bman at sqr 1 holds wman at 5)
evalkingdcorner = $0f	; Value for weaker sides king in double corner in endgame

;jfindbestalpha:         ; (PET:JS=$1800)
;  jmp findbestalpha

;jgenlegalmoves:
;  jmp genlegalmoves

evalmaterial:           ; Eval EVALTOT for material & exchange factors  (PET:E0=$1806)
  lda #$00
  sta zEvalTot          ; Lo = 0, later for time/brd eval
  lda valwhite
  beq evalm1            ; Zero white pieces left, White loss.:
  ldy valblack
  beq evalm2            ; Zero black pieces left, Black loss.:
  sec 
  sbc valblack		; Calc Material (white-black) (man=2,k=3)
  beq evalm3            ; Dead heat.:
  clc                   ; Work out exchange factors.. (encourage exchanges when winning, discourage when losing)
  adc #$20
  asl 
  sta zEvalTot+1
  tax 
  tya 
  clc 
  adc valwhite
  cpx #$40
  bcc evalm4
  eor #$ff
evalm4:
  asl 
  asl 
  asl 
  rol zEvalTot+1         ; Save exchange factors
  sta zEvalTot
  rts
   
evalm1:			; WIN/LOSS/EQUAL jump points. (PET:E1=$1834)
  lda #$00
  .byte $2c		; Classic 6502 kludge - branch to following LDA #$FF used elsewhere, $2C BIT instruction "ignores" it on this line.
evalm2:
  lda #$ff
  .byte $2c		; and again..		
evalm3:
  lda #$80
  sta zEvalTot+1
  rts


   
evalposition:		; (PET:ED=$183f)   (Note code different from PET-ORAC-2.15)

.if SHOW_MOVE_INFO		; Increment node count..
  inc nodecount
  bne evalnodedone
  inc nodecount+1
  bne evalnodedone2
  inc nodecount+2
evalnodedone2:
  jsr nodeshowhi    ; Show hi part node count
evalnodedone:
.endif

  ldy ctime		; running eval of sqr vals coming up the tree.. (Y holds lo byte eval)
  lda valwhite
  clc
  adc valblack
  cmp #$10		; tot mat 16 (ie 8 men)
evalpos01:
  ldx nkings
  bcs evalpos02		; >8men, eva; bridges..
  bne evalpos04		; Kings on board, skip bridge etc..
evalpos02:              ; White Bridge  (sq 30,32)
  lda tboard+6
  and tboard+2
  cmp # pwhiteman
  bne evalpos03
  tya 
  adc # evalbridge-1	; Add bridge value (carry is set, so 1 less)
  tay 
evalpos03:		; Black Bridge (sqrs 1,3)
  lda tboard+57
  and tboard+61
  cmp # pblackman
  bne evalpos04
  tya 
  sbc # evalbridge
  tay 
evalpos04:			; Black in Dog-hole (sq 32,28) 
  lda tboard+6
  eor tboard+15
  cmp # pwhite_xor_black	; Unique val when xor white & black man vals.
  bne evalpos05
  tya 
  adc # evaldoghole-1
  tay 
evalpos05:			; White in Dog-hole (sq 1,5) (PET:$1873)
  lda tboard+57
  eor tboard+48
  cmp # pwhite_xor_black
  bne evalpos06
  tya 
  sbc # evaldoghole
  tay 
evalpos06:
  txa 
  beq evalposdone		; No kings, done..
  ldx #$03
  lda valblack
  cmp valwhite
  beq evalposdone		; Even material, done..
  bcs evalpos07			; Black > White, x=3
  ldx #$fd			; Black < White, x=$fd

evalpos07:			; Award bonus to weaker side if he has free single king in double corner.
  txa 
  sec 
  sbc tboard+6
  cmp tboard+15
  beq evalpos08
  txa 
  sec 
  sbc tboard+48
  cmp tboard+57
  bne evalposdone
evalpos08:
  tya 
  cpx #$80
  bcc evalpos09
  sbc # (evalkingdcorner*2)+1
evalpos09:
  adc # evalkingdcorner
  tay 

evalposdone:
  tya				; Add tot evals (y) into main (zEvalTot)
  clc 
  adc zEvalTot
  .if USE_RANDMODE	; Randomise low bits of eval with random value..
    eor rand_val
  .endif
  sta zEvalTot
  bcc evalpos11
  inc zEvalTot+1
evalpos11:
  rts
   
;patchmakemove18bb:			; (PET:$18bb)
;  txa 
;  clc 
;  adc dirhoriz
;  clc 
;  adc dirvert
;  tax 
;  rts

evaluatepos:            ; Evaluate all factors, return zEvalTot (PET:EV=$1900)
  jsr evalmaterial
  jmp evalposition	

;------------------------------------------------------------------------------
; Generate legal move list.

genlegalmoves:			; Gen legal moves (PET:L0=$197c,page d10)
  jsr gensaveold
  lda #$00
  sta takeflag
  sta cmobil
  sta nmoves
  lda mvoption
  bmi genlegal3
  
  ldx #$1f
genlegal2:			; Scan all 32 board squares.
  stx mvgenindex
  lda tmvorder,x		; Convert index (0-31) to actual board square 
  sta zmove
  jsr gen1pmoves		; Gen moves for square (zmove)
  ldx mvgenindex
  dex 
  bpl genlegal2

geninitply:			; initialise ply data (PET:L3=$19a2, page d10b)
  ldy cply
  lda nmoves
  sta tnomoves,y 
  clc 
  adc interest
  sta interest
  sta tinterest,y 
  lda curcolor
  sta tcolor,y 
  lda zPtrMov
  sta tmoveptrlo,y 
  lda zPtrMov+1
  sta tmoveptrhi,y 
geninitply1:
  ldx #$00
  txa 
  sta tcurmove,y 
  bit curcolor
  bpl geninitply2
  dex 
  txa 
geninitply2:
  sta tmoveevallo,y 
  sta tmoveevalhi,y 
  lda nkings
  sta tkings,y 
  rts
   
genlegal3:					; (PET:L1=$19df)
  and #$3f
  sta zmove
  lda #$01
  sta takeflag
  jsr gen1pmoves
  lda nmoves
  ;bne geninitply
  jmp geninitply
  ;brk 				; ???
  
  
gensaveold:				; Init legal gen (PET:LA=$19f1)
  ldy cply
  lda ctime
  sta oldtime
  sta ttime,y 
  lda valblack
  sta oldblack
  sta tblack,y 
  lda valwhite
  sta oldwhite
  sta twhite,y 
  lda zPtrMov
  clc 
  adc #<(tdirstack - tmovestack)		; Bug - was const on PET, should be calc'd thus.. (PET:$54)
  sta zPtrDir
  lda zPtrMov+1
  adc #>(tdirstack - tmovestack)		;  (PET:$01)
  sta zPtrDir+1
gensave1:
  ldx #$00
  stx nmoves
  lda curcolor
  beq gensave2
  dex 
gensave2:
  stx bestval
  stx bestval+1
  rts
   
genforsqr:				; Generate and count legal moves (PET:J0=$1a2c, page d11)
  lda #$00
  .byte $2c				; Skip instruction - classic 6502 BIT trick..
gen1pmoves:				
  lda #$80
  sta mvgenflag
  ldx zmove
  lda tboard,x 
  beq gensqrexit			; empty sqr, skip..
  eor curcolor		
  bmi gensqrexit			; skip piece if not side-to-play colour
  ldx curcolor
  bmi gensqrl4
gensqrl9:
  ldx #$08
  .byte $2c				; Skip instruction - classic 6502 BIT trick..
gensqrl4:
  ldx #$f8
  stx dirvert
  lda #$ff
  sta dirhoriz
gensqrl7:				; (PET:L7=$1a52)
  bit mvgenflag
  bpl gensqrj1
  jsr gencheckamove
  beq gensqrl5
  inc cmobil
  jsr genmvonbrd
gensqrl5:
  inc dirhoriz
  bne gensqrl6
  inc dirhoriz
  bne gensqrl7
gensqrl6:
  ldx zmove
  lda tboard,x 
  lsr 
  bcc gensqrexit
  lda curcolor
  eor dirvert
  bmi gensqrexit
  lda curcolor
  bmi gensqrl9
  bpl gensqrl4
gensqrexit:
  rts
   
gensqrj1:				; (PET:J1=$1a84)
  jsr gencheckamove
  cpy #$02
  bne gensqrl5
  inc mvgenflag
  bne gensqrl5

gencheckamove:			;  (PET:G0=$1a90, page 12)
  ldx zmove
  jsr gendiradd
  bne gencheckret0
  lda tboard,x 
  beq gencheckret1
  eor curcolor
  bpl gencheckret0
  jsr gendiradd
  bne gencheckret0
  lda tboard,x 
  beq gencheckret2
gencheckret0:
  ldy #$00
  rts
   
gencheckret1:
  ldy #$01
  rts
   
gencheckret2:
  ldy #$02
  rts
   
gendiradd:				; Add dir to x (PET:GL=$1ab4)
  txa 
  stx zMem7
  clc 
  adc dirhoriz
  sta ztemp
  eor zMem7
  and #$f8
  bne gendiradd1
  lda ztemp
  clc 
  adc dirvert
  tax 
  eor ztemp
  and #$c7
gendiradd1:
  rts
   
genmvonbrd:			; Gen move acc to (mvoption) (PET:GM=$1acf)
  lda takeflag
  cpy #$02
  beq genmvg5
  tax 
  beq genmvg6
  rts
   
genmvg5:
  tax 
  bne genmvg6
  inx 
  stx takeflag
  jsr gensave1
genmvg6:
  jsr makemove
  cpx #$08
  bcc genmvg8
  cpx #$38
  bcc genmvg7
genmvg8:
  lsr 
  bcs genmvg7
  cmp #$40
  lda #$03
  inc valwhite
  bcc genmvg9
  lda #$fd
  inc valblack
  dec valwhite
genmvg9:
  sta tboard,x 

genmvg7:		; (PET:G7=$1b06)
  txa 
  pha 
  jsr evaluatepos
  pla 
  tax 
  jsr unmove
  lda zEvalTot
  ldx curcolor
  beq genmvga
  sec 
  sbc bestval
  lda zEvalTot+1
  sbc bestval+1
  bcc genmvgb
  bcs genmvgc
genmvga:
  clc 
  sbc bestval
  lda zEvalTot+1
  sbc bestval+1
  bcc genmvgc
genmvgb:
  lda zmove
  sta bestmove
  clc 
  lda dirvert
  adc dirhoriz
  sta bestdir
  lda zEvalTot
  sta bestval
  lda zEvalTot+1
  sta bestval+1
genmvgc:
  lda mvoption
  cmp #$02
  bne genmvgd
genmvgf:
  rts
   
genmvgd:		; (PET:GD=$1b50)
  ldx jumpflag
  beq genmvge
  cmp #$01
  bne genmvge
  lda takeflag
  beq genmvgf
genmvge:
  jmp putmove2tabp5

;------------------------------------------------------------------------------
; Make a move on board during search

makemove:		; Do move (zmove) dir (dirvert,dirhoriz) (PET:M0=$1b61) (Note following rewritten since PET-ORAC-2.15)
  lda nkings
  sta oldkings
  ldx zmove
  lda tboard,x 
  sta oldpiece
  lsr 
  php 
  lda #$00
  sta tboard,x 
  ; jsr patchmakemove18bb
  txa 
  clc 
  adc dirhoriz
  clc 
  adc dirvert
  tax 
  
  lda tboard,x 
  sta oldtaken
  bne makemov06
  plp 
  bcs makemov04
makemov01:
  bit curcolor
  bmi makemov02		; black move..
  ldy zmove
  lda ctime		; Update "time" weight eval for moving white man..
  sec 
  sbc tmancent,y	
  clc 
  adc tmancent,x 
  sta ctime
  clc 
  bcc makemov03
makemov02:
  lda zmove
  eor #$3f
  tay 
  txa 
  eor #$3f
  tax 
  lda ctime		; Update "time" weight eval for moving black man..
  sec 
  sbc tmancent,x 
  clc 
  adc tmancent,y 
  sta ctime
  txa 
  eor #$3f
  tax 
  clc 
makemov03:				; (PET:$1bb6)
  bcc makemov16
makemov04:
  bcs makemov13
makemov05:
  bcc makemov01
makemov06:
  tay 
  bpl makemov07
  clc 
  adc valblack
  sta valblack
  bcs makemov08
makemov07:
  eor #$ff
  sec 
  adc valwhite
  sta valwhite
makemov08:					; (PET:$1bd1)
  lda #$00
  sta tboard,x 
  tya 
  lsr 
  lda ctime
  bcc makemov10
  dec mvkiller
  bit curcolor
  bmi makemov09
  clc 
  adc tkingcent,x 
  bcc makemov12
makemov09:
  sec 
  sbc tkingcent,x 
  bcs makemov12
makemov10:
  bit curcolor
  bpl makemov11
  sec 
  sbc tmancent,x 
  bcs makemov12
makemov11:
  txa 
  eor #$3f
  tay 
  clc 
  lda ctime
  adc tmancent,y 
makemov12:				; (PET:$1c07)
  sta ctime
  ; jsr patchmakemove18bb
  txa 
  clc 
  adc dirhoriz
  clc 
  adc dirvert
  tax 
  
  plp 
  bcc makemov05
makemov13:
  lda ctime			; Update "time" weight eval for moving king..
  sec 
  ldy zmove
  bit curcolor
  bmi makemov14
  sbc tkingcent,y 
  clc 
  adc tkingcent,x 
  bcc makemov15
makemov14:
  sbc tkingcent,x 
  clc 
  adc tkingcent,y 
makemov15:
  sta ctime
makemov16:
  lda oldpiece
  sta tboard,x 
  rts
   
;------------------------------------------------------------------------------
; Undo a move during search

unmove:			; Unmove (PET:U0=$1c37)
  lda #$00
  sta tboard,x 
  ldx zmove
  lda oldpiece
  sta tboard,x 
  ldy oldtaken
  beq unmoveu1
  txa 
  clc 
  adc dirhoriz
  clc 
  adc dirvert
  tax 
  tya 
  sta tboard,x 
unmoveu1:
  lda oldtime
  sta ctime
  lda oldblack
  sta valblack
  lda oldwhite
  sta valwhite
  lda oldkings
  sta nkings
  rts
   
putmove2tabp5:			; Put move into stack-table (PET:P0=$1c6b, page d17)
  ldy #$00
  jmp putmove2tabp1
putmove2tabp5p5:
  sty ztemp
  ldy nmoves
  beq putmove2tabp7
putmove2tabp2:
  dey 
  lda (zPtrMov),y
  iny 
  sta (zPtrMov),y
  dey 
  lda (zPtrDir),y
  iny 
  sta (zPtrDir),y
  dey 
  lda tspotevallo,y 
  iny 
  sta tspotevallo,y 
  dey 
  lda tspotevalhi,y 
  iny 
  sta tspotevalhi,y 
  dey 
  cpy ztemp
  bpl putmove2tabp2
putmove2tabp7:
  ldy ztemp
  jmp putmove2tabp3

putmove2tabp1:				; (PET:P1=$1c9d)
  ldx nmoves
  beq putmove2tabp3
  lda curcolor
  beq putmove2tabp4
  sec 
  lda zEvalTot
  sbc tspotevallo,y 
  lda zEvalTot+1
  sbc tspotevalhi,y 
  bcc putmove2tabp5p5
  bcs putmove2tabp6
putmove2tabp4:
  clc 
  lda zEvalTot
  sbc tspotevallo,y 
  lda zEvalTot+1
  sbc tspotevalhi,y 
  bcs putmove2tabp5p5
putmove2tabp6:
  iny 
  cpy nmoves
  bcc putmove2tabp1
putmove2tabp3:				; (PET:P3=$1cc9, page d18)
  lda zmove
  sta (zPtrMov),y
  lda dirvert
  clc 
  adc dirhoriz
  sta (zPtrDir),y
  lda zEvalTot
  sta tspotevallo,y 
  lda zEvalTot+1
  sta tspotevalhi,y 
  inc nmoves
  rts

;------------------------------------------------------------------------------
; Recursively search for a move

findbestalpha:   		; (PET:S0=$1ce4, page d20)
  jsr genlegalmoves
  lda takeflag			; Force examine capture seq.:
  bne contsearch		; BUG?? should check UPPERPLY cut at findbestq5?
  lda cply			; See if cut-off reached
  clc
  tax
  adc interest			; Experiment with LDA interest here (deep capture)
  cmp iqcut
  bcc findbestq5
  cpx lowerply
  bcc contsearch
finddocut:			; Ok, Cut-off reached.: (PET:Q7=$1cfe)
  jmp docutoff
findbestq5:
  cpx upperply			; Absolute MAX_PLY depth, dont go deeper.
  bcs finddocut
contsearch:
  lda nmoves
  beq finddocut
  jsr shortlook

findbestq2:			; Main search move re-loop point
  jsr execmove

  ldy cply
  bne l1fef             ; Not lowest level (ply 1), skip
  lda tcurmove,y        ; Display Move #
  clc
  adc #$31
  sta pVDU_SIDE+1
 .if SHOW_MOVE_INFO  
   jsr showinfo
 .endif
  ldy cply
l1fef:
  lda curcolor
  tax

  dey 
  bmi findbests7	; ply<0, end search..
  cmp tcolor,y 		
  beq findbests7
findbests5:
  dey 
  bmi findbests4
  cmp tcolor,y 
  bne findbests5
findbests4:
  iny 
  tax 
  lda bestval
  cpx #$00
  beq findbests3
  sec
  sbc tmoveevallo,y 
  lda bestval+1
  sbc tmoveevalhi,y 
  bcc findbestqj
  bcs findbests7

findbests3:
  clc 
  sbc tmoveevallo,y 
  lda bestval+1
  sbc tmoveevalhi,y 
  bcs findbestqj
findbests7:			; (PET:S7=$1d4b, page d21)
  ldy cply
  lda bestval
  cpx #$00
  beq findbests8
  sec 
  sbc tmoveevallo,y 
  lda bestval+1
  sbc tmoveevalhi,y 
  bcc findbestq0
  bcs findbestq1
findbests8:
  clc 
  sbc tmoveevallo,y 
  lda bestval+1
  sbc tmoveevalhi,y 
  bcc findbestq1
findbestq0:
  jsr findbestqj
  jmp findbestq1

findbestqj:			; Best found, save best move/eval (PET:QJ=$1d75)

  lda bestval
  ldy cply

  sta tmoveevallo,y 
  lda bestval+1
  sta tmoveevalhi,y 
  lda zmove
  sta tbestmove,y 
  lda dirvert
  sta tbestdir,y 
  rts				
   
findbestq1:			; OK, next move & loop?	(PET:Q1=$1d90)
  lda tcurmove,y 
  clc 
  adc #$01
  sta tcurmove,y 
  cmp tnomoves,y		; if (curmove < nomoves) then loop
  bcs findbests6
  jmp findbestq2

findbests6:			; Drop out of move loop, return best "alpha"
  ldy cply
  lda tmoveevallo,y 
  sta bestval
  lda tmoveevalhi,y 
  sta bestval+1
  rts
   
docutoff:		; (PET:S1=$1db1)
  lda cmobil            ; Out of moves ?
  bne docuroffq3             ; No, CUT back to last level
  ldx #$00              ; No mobile, so LOST.:
  lda curcolor
  beq docuroffq4
  dex 
docuroffq4:	
  stx bestval
  stx bestval+1
  rts
   
docuroffq3:
  rts

;------------------------------------------------------------------------------
; Use a "short look ahead" mini search to sort moves into good order
   
shortlook:		; Perform a "short look-ahead", to improve move ordering for main search. (PET:B0=$1dc6, page d22)
  lda nmoves
  cmp #$02		; If mobility<2 moves, dont do short look
  bcc shortb1
  cmp #$18
  bcs shortb1		; If mobility>=24 moves, dont do short look
  lda alphakill
  beq shortb5		; Not already doing short look, so try it..
shortb1:
  rts
   
shortb5:			; (PET:B5=$1dd7)
  lda iqcut			; calc reduced iq for short look ahead
  sec 
  sbc interest
  cmp #$18
  bcc shortb1
  lda iqcut
  pha 
  clc 
  adc interest
  lsr 
  clc 
  adc interest
  lsr 
  sta iqcut
  lda #$01
  sta alphakill		; ensure no more recursive short look aheads while doing this on.
shortb2:
  jsr execmove		; do short look
  ldy cply
  ldx tcurmove,y
  lda zmove
  sta tabbmove,x	; Save best move/dir/eval
  lda dirvert
  sta tabbdir,x 
  lda bestval
  sta tabbvallo,x 
  lda bestval+1
  sta tabbvalhi,x 
  inx 
  txa 
  sta tcurmove,y 
  cmp tnomoves,y 
  bcc shortb2
  lda #$00
  sta dirhoriz
  sta alphakill
  sta nmoves
shortb4:
  ldy nmoves
  lda tabbmove,y 		; Get best move/dir/eval
  sta zmove
  lda tabbdir,y 
  sta dirvert
  lda tabbvallo,y 
  sta zEvalTot
  lda tabbvalhi,y 
  sta zEvalTot+1
  jsr putmove2tabp5
  lda nmoves
  ldy cply
  cmp tnomoves,y 
  bcc shortb4
  jsr geninitply1
  pla 
  sta iqcut
  rts

;------------------------------------------------------------------------------
; Execute move during search

execmove:		; Execute move during search, go deeper.. (PET:X0=$1e5b, page d25)
  ldy cply
  lda #$00
  sta mvoption
  lda tcurmove,y 
  tay 
  lda (zPtrMov),y
  sta zmove
  ldx #$08
  lda (zPtrDir),y
  bpl execx3
  ldx #$f8
execx3:
  stx dirvert
  sec 
  sbc dirvert
  sta dirhoriz
  jsr makemove
  stx zmove
  ldy cply
  lda tnomoves,y 
  clc 
  adc zPtrMov
  sta zPtrMov
  bcc execx4
  inc zPtrMov+1
execx4:					; (PET:x4=$1e91,page d26)
  lda oldkings
  sta tkings,y 
  lda oldpiece
  sta tpiece,y 
  lda oldtaken
  sta ttakenpiece,y 
  beq execx5
  jsr genforsqr
  lda mvgenflag
  beq execx5
  lda zmove
  ora #$80
  sta mvoption
  bne execx6
execx5:
  lda curcolor
  eor #$80
  sta curcolor
execx6:
  ldx zmove
  lda oldpiece
  cpx #$08
  bcc execx7
  cpx #$38
  bcc nextply
execx7:					; (PET X7=$1ec8)
  lsr 
  bcs execxb
  cmp #$40
  lda #$03
  inc valwhite
  bcc execx9
  lda #$fd
  inc valblack
  dec valwhite
execx9:
  sta tboard,x 
execxb:
  jmp nextply

termandeval:				; (PET X2=$1ee2)
  lda curcolor
  eor #$80
  sta curcolor
  jsr evaluatepos
  lda curcolor
  eor #$80
  sta curcolor
  lda zEvalTot
  sta bestval
  lda zEvalTot+1
  sta bestval+1
  rts
   
nextply:				; ; (PET:X8=$1f00)
  inc cply
  jsr findbestalpha
  dec cply				; Down 1 ply
  ldy cply				; Pull game-state variable off stack
  lda tcolor,y 
  sta curcolor
  lda tmoveptrlo,y 
  sta zPtrMov
  clc 
  adc # <(tdirstack - tmovestack)		; Bug - was const on PET, should be calc'd thus.. (PET:$54)
  sta zPtrDir
  lda tmoveptrhi,y 
  sta zPtrMov+1
  adc # >(tdirstack - tmovestack)		;  (PET:$01)
  sta zPtrDir+1
  lda ttime,y 
  sta ctime
  lda tinterest,y 
  sta interest
  lda tblack,y 
  sta valblack
  lda twhite,y 
  sta valwhite
  lda tkings,y 
  sta nkings
  lda tcurmove,y 
  tay 
  lda (zPtrMov),y
  sta zmove
  tax 
  lda (zPtrDir),y
  sta dirvert
  ldy cply
  lda tpiece,y 
  sta tboard,x 
  txa 
  clc 
  adc dirvert
  tax 
  lda ttakenpiece,y 
  sta tboard,x 
  beq execxa
  txa 
  clc 
  adc dirvert
  tax 
  lda #$00
  sta tboard,x 
execxa:		
  rts

;-----------------------------------------------------------------------------------
; end of ORAC checkers playing engine code.
;-----------------------------------------------------------------------------------

.if SHOW_MOVE_INFO		; Code to show best move, eval, node count etc..

byte2hex:		; Convert byte to hex in (X;A)
  pha
  lsr
  lsr
  lsr
  lsr
  jsr nib2hex		; Connvert hi nibble
  tax
  pla
  and #$0f
			; Drop through and get last nibble in Acc.
nib2hex:		; Convert hex nibble to hex asc (0-9,A-F)
  cmp #10
  bcc nib2hex2
  adc #198   ; for PET screen code  (A=1..)
  ; adc #6   ; for std ascii  (A=64..)
nib2hex2:
  adc #48
  rts

showinfo:			; Show best move, score,node count
  lda tbestmove
  bmi nodeshow
  sta zPtrGen
  jsr coord2brd			; Convert brd index (0-63) to trad numbering (1-32)
  jsr convbyte2dec
  stx pVDU_BESTMOV		; and put on screen (pVDU+793)
  sta pVDU_BESTMOV+1
  lda tbestmove			; Now print dest
  clc
  adc tbestdir
  sta zPtrGen
  jsr coord2brd			; Convert to trad numbering (1-32)
  jsr convbyte2dec
  stx pVDU_BESTMOV+2		; and put dest on screen
  sta pVDU_BESTMOV+3
				; Print best eval (hex XXXX)
  lda tmoveevalhi
  jsr byte2hex
  stx pVDU_BESTVAL
  sta pVDU_BESTVAL+1
  lda tmoveevallo
  jsr byte2hex
  stx pVDU_BESTVAL+2
  sta pVDU_BESTVAL+3
  
nodeshow:			; Show node count
  lda nodecount
  jsr byte2hex
  stx pVDU_NODES+4
  sta pVDU_NODES+5
nodeshowhi:			; Show node count (hi 2 bytes only)
  lda nodecount+1
  jsr byte2hex
  stx pVDU_NODES+2
  sta pVDU_NODES+3
  lda nodecount+2
  jsr byte2hex
  stx pVDU_NODES
  sta pVDU_NODES+1
  rts
 
.endif			; // SHOW_MOVE_INFO

;------------------------------------------------------------------------------

; CBM64 Specific code..

.if IS_C64

.if USE_CLOCK
io_clockoff:
  sei
  lda # <(romirq)
  sta vectirq
  lda # >(romirq)
  sta vectirq+1
  cli
  rts
.endif

io_initialise:		; Initialise CBM64 specific stuff
  ; jsr romrestorevectors
  
.if USE_HIRES
   jsr init_graphics    ; Setup hi res piece characters
.endif  
  ; jsr clr128fast
  
io_color_set:
  lda color_back	; Setup c64 screen/textborder colours.
  sta vicborder
  sta vicback		; Background colour
  
io_fill_col:		; Set screen ink color
  lda color_ink
  ldx #$00
io_fill1:
  sta pcolor,x
  sta pcolor+$100,x
  sta pcolor+$200,x
  sta pcolor+$300,x 
  inx 
  bne io_fill1
  rts

io_getkey:			; Get a key.. 
  jsr kerngetin			; Read a key from keybuf ($ffe4)
  ;  jsr c128patch
.if USE_COLOR  
  tax
  sec
  sbc #$85			; Func key?
  cmp #4
  bcc io_get1			; Func key.. (F1/3/5/7)
  txa				; reget A=key hit..
  rts
  
io_get1:  
  tax				; Index 0..3 color to change..
  inc color_table,x
  lda color_table,x
  and #15
  sta color_table,x
  jsr io_color_set
  lda #13
.endif			; (USE_COLOR)  
  rts
    
io_wait_jiffy2:					; wait for 2 jiffys..
  jsr io_wait_jiffy
io_wait_jiffy:					; wait for a jiffy, 1/60 sec..
  lda zp_jiffy_low
io_wait_jiffy1:
  cmp zp_jiffy_low
  beq io_wait_jiffy1
  rts


.if USE_SOUND
io_sound:				; Make a simple sound, using SID.
  lda #$00
  ldx #$18
io_sound1:
  sta sid,x 
  dex 
  bpl io_sound1
  lda #$0b
  sta sid+5
  lda #$15
  sta sid+$18
  lda #$19
  sta sid+1
  lda #$b1
  sta sid+2
  lda #$11
  sta sid+4
io_sound_off:
  rts
.endif
		

.if USE_HIRES	; Use hi-res (UDG at $3000)

init_graphics:		; Setup hi res piece characters
  sei			; Prevent interrupts during copying of $D000 CHRSET ROM..
  lda C64memctl
  and # $fb		; Enable chr set at $D000 (BIT 2: poke 1, peek(1) and 251)
  sta C64memctl

  ldx #8		; Copy over (16x256page) C64 ROM char set at $D000 to UDG RAM at $3000.. 
  jsr io_copy_hires	; Copy char sets, init hires gfx
  lda C64memctl		; Reenable interrupt, back to normal..
  ora # $04		
  sta C64memctl
  cli  
  lda # $1d
  sta VICmem		; Set VDU=$0400,CHRSET=$3000 (Normally $15 for Default VDU=$0400,CHRSET=$D000)
  rts


.endif		; (USE_HIRES)

.endif	 	; IS_C64

;------------------------------------------------------------------------------
; PET specific code

.if IS_PET

.if USE_CLOCK
io_clockoff:
  sei
  lda # <(romirq)
  sta vectirq
  lda # >(romirq)
  sta vectirq+1
  cli
  rts
.endif

io_initialise:		; Initialise PET specific stuff
  .if USE_CLOCK
   jsr io_clockoff
  .endif
  
  lda #$0c
  sta PET_CASE		; PET:Upper case/graphics
  rts
  
io_getkey:
  jmp kerngetin

io_wait_jiffy2:					; wait for 2 jiffys..
  jsr io_wait_jiffy
io_wait_jiffy:					; wait for a jiffy, 1/60 sec..
  lda zp_jiffy_low
io_wait_jiffy1:
  cmp zp_jiffy_low
  beq io_wait_jiffy1
  rts

.if USE_SOUND
io_sound:				; Make a simple beep sound (for moves etc).
  ldx #$10				; Original simple PET sound routine..
  stx PET_SOUND_e84b
  dex
  stx PET_SOUND_e84a
  ldx #$22
  stx PET_SOUND_e848
  .if (USE_SOUND = 2)		; Long beep or click?
    rts
  .else
    jsr io_wait_jiffy2		; short click
  .endif
io_sound_off:
  ldx #$00			; kill sound on pet
  stx PET_SOUND_e84b
  rts
.endif			; (USE_SOUND)

.endif 			; IS_PET

;------------------------------------------------------------------------------
; C16/+4 specific code

.if IS_C16

.if USE_CLOCK
io_clockoff:
  sei
  lda # <(romirq)
  sta vectirq
  lda # >(romirq)
  sta vectirq+1
  cli
  rts
.endif

io_initialise:		; Initialise C16 specific stuff
  .if USE_CLOCK
   jsr io_clockoff
  .endif
.if USE_HIRES
   jsr init_graphics    ; Setup hi res piece characters
.endif  


io_color_set:  
  lda color_back	; Setup screen/textborder colours.
  jsr io_color_translate	; Translate color from index to actual colour to use. (X & Y must be preserved)
  sta TED_VDUCOL	 	; Set screen black
  sta TED_BORDER	; Set border black
  
io_fill_col:		; Set screen ink color
  lda color_ink
  jsr io_color_translate	; Translate color from index to actual colour to use. (X & Y must be preserved)
  ldx #$00
io_fill1:
  sta pcolor,x
  sta pcolor+$100,x
  sta pcolor+$200,x
  sta pcolor+$300,x 
  inx 
  bne io_fill1
  rts

io_color_trantab:		; Table to Pick color from C16 pallete of (0-127)
  .byte 0,113,98,99,68,69,70,60,59,1,2,3,4,5,6,7
io_color_temp:
  .byte 0
  
io_color_translate:		; Translate color from index to actual colour to use. (C16) (X & Y must be preserved)
  stx io_color_temp
  tax
  lda io_color_trantab,x
  ldx io_color_temp
  rts

io_getkey:			; Get a key.. 
  jsr kerngetin			; Read a key from keybuf ($ffe4)
  
.if USE_COLOR  			; Allow user to change colour
  tax
  sec
  sbc #$04			; Func key?
  cmp #4
  bcc io_get1			; Func key.. (F1/3/5/7)
  txa				; reget A=key hit..
  rts
  
io_get1:  
  tax				; Index 0..3 color to change..
  inc color_table,x
  lda color_table,x
  and #15
  sta color_table,x
  jsr io_color_set
  lda #13
.endif 				; (USE_COLOR)
  rts
 
io_wait_jiffy2:					; wait for 2 jiffys..
  jsr io_wait_jiffy
io_wait_jiffy:					; wait for a jiffy, 1/60 sec..
  lda zp_jiffy_low
io_wait_jiffy1:
  cmp zp_jiffy_low
  beq io_wait_jiffy1
  rts

.if USE_SOUND

io_sound:				; Make a simple beep sound (for moves etc).
  lda #3
  sta TED_VOICE2LO
  sta TED_VOICE2HI
  lda #$2f			; Enable voice2 tone, vol=15
  sta TED_VOL
  .if USE_SOUND = 2		; Long beep or click?
    rts
  .else
    jsr io_wait_jiffy2		; short click
  .endif
io_sound_off:
  lda #0
  sta TED_VOL
  rts

.endif			; (USE_SOUND)

.if USE_HIRES	; Use hi-res (UDG at $3000)

init_graphics:		; Setup hi res piece characters
  sei			; Prevent interrupts during copying of $D000 CHRSET ROM..
  ldx #8		; Copy over (4x256page) C16 ROM char set at $D000 to UDG RAM at $3000.. 
  jsr io_copy_hires	; Copy char sets, init hires gfx

  lda #$30		; Set chrset base to $3000
  sta TED_CHRBASE		
  lda #TED_MEMCTL_GFX	; Set chrset to RAM, bit 2 off (192)
  sta TED_MEMCTL		; TED $FF12
  cli  
  rts		
  
.endif			; (USE_HIRES)

.endif 			; IS_C16

;------------------------------------------------------------------------------
; VIC20 specific code

.if IS_V20

.if USE_CLOCK
io_clockoff:
  sei
  lda # <(romirq)
  sta vectirq
  lda # >(romirq)
  sta vectirq+1
  cli
  rts
.endif

io_color_set:
.if USE_COLOR  
  lda color_back	; Setup c64 screen/textborder colours.
  asl
  asl
  asl
  asl
  ora color_back
  ora #8
  sta V20_BACKCOL  	; Set background/border colour
  
io_fill_col:		; Set screen ink color
  lda color_ink
  ldx #$00
io_fill1:
  sta pcolor,x
  sta pcolor+$100,x
  sta pcolor+$200,x
  sta pcolor+$300,x 
  inx 
  bne io_fill1
.endif			; (USE_COLOR)  
  rts
 
io_getkey:
  jsr kerngetin
.if USE_COLOR  
  tax
  sec
  sbc #$85			; Func key?
  cmp #4
  bcc io_get1			; Func key.. (F1/3/5/7)
  txa				; reget A=key hit..
  rts
  
io_get1:  
  tax				; Index 0..3 color to change..
  inc color_table,x
  lda color_table,x
  and #7
  sta color_table,x
  jsr io_color_set
  lda #13
.endif			; (USE_COLOR)  
  rts


io_wait_jiffy2:					; wait for 2 jiffys..
  jsr io_wait_jiffy
io_wait_jiffy:					; wait for a jiffy, 1/60 sec..
  lda zp_jiffy_low
io_wait_jiffy1:
  cmp zp_jiffy_low
  beq io_wait_jiffy1
  rts

.if USE_SOUND

io_sound:				; Make a simple beep sound (for moves etc).
  lda #220
  sta V20_VOICE3
  lda #15
  sta V20_VOL
  .if USE_SOUND = 2		; Long beep or click?
    rts
  .else
    jsr io_wait_jiffy2		; short click
  .endif
io_sound_off:
  lda #0
  sta V20_VOL
  rts

.endif			; (USE_SOUND)

.endif 			; IS_V20

;=====================================================================================
; NOTE - code/data placed after here can be reused once we jump to (newgame/mainloop)
;=====================================================================================

PHEAP = * + 4		; Heap for search-data tables start here.

.if USE_HIRES

io_copy_hires:		; Copy hi-res piece sets to RAM, X= #pages of chr set to copy (4=1K, 8=2K)  
  ldy #0
init_gra1:
  lda PTR_ROMSET,y	; SELF-MODIFYING CODE!
  sta PTR_CHRSET,y
  iny
  bne init_gra1
  inc init_gra1+2	; Next 256byte page.. Warning - selfmod pointers! (Call this ONCE)	
  inc init_gra1+5	
  dex
  bne init_gra1
			; Now copy orac hires piece data into chars CUSTOMCHR (193-208)
  ldx # SET_HEIGHT * 64 		; Copy over piece set data (wm/wk/bm/bk)
init_gra2:
  lda orac_hires_pieces-1,x
  sta PTR_CHRSET + (CUSTOMCHR * 8)-1,x
  dex
  bne init_gra2

  ldx # NO_UDG_CHR*8-1		; Copy extra UDG user characters..
init_gra3:
  lda hires_chars,x
  sta PTR_CHRSET + (UDG_CHARS * 8),x
  dex
  bpl init_gra3
  rts		
.endif 			; (USE_HIRES)

;------------------------------------------------------------------------------

.if IS_V20		; Place VIC20 initialisation code here, reuse once running..

io_initialise:		; Initialise V20 specific stuff
  .if USE_CLOCK
   jsr io_clockoff
  .endif
  lda #8
  sta V20_BACKCOL  	; Set background/border colour
  lda #5		
  jsr kernchrout
  lda #147		; CLS
  jsr kernchrout

.if USE_HIRES	; Use hi-res (UDG at $3000)
  jsr io_fill_col

init_graphics:		; Setup hi res piece characters
  
  ldx #8		; Copy over (8x256page) VIC ROM char set at $8000 to UDG RAM at $1200.. 
  jsr io_copy_hires	; Copy char sets, init hires gfx

  ldx V20_HPOS		; Adjust screen horizontal centering
  dex
  dex
  stx V20_HPOS
  ldx V20_VPOS		; Adjust screen vertical centering
  dex
  dex
  stx V20_VPOS
  
  lda # VDUX
  sta V20_HSIZE		; bits 0-6 set # of columns, bit 7 part video matrix address
  lda # VDUY*2
  sta V20_VSIZE		; bits 1-6 set # of rows, bit 0 for 8x8 or 8x16 chars
  lda # $cd
  sta V20_VIDADD	; Set VDU=$1000,CHRSET=$1200 (Normally $c0 (v20+8k) for Default VDU=$1200,CHRSET=$8000)

.endif		; (USE_HIRES)
  rts
  
.endif 			; IS_V20

;-----------------------------------------------------------------------------------

runprog:   	; MAIN ORAC initialisation routine, called by basic SYS xxxx (PET:L0=$0d76,page i19)
		; NOTE - this code can be reused once we jump to  mainloop
  cld
  sei
  ldx #$f8
  txs

  lda #0			; Clear zero page storage  (NOTE: set to 1=problem. engine bug lurking!)
  ldx #(zpEnd-zpStart+1)
runp1:
  sta zpStart-1,x
  dex
  bne runp1

  jsr io_initialise		; Initialise computer specific variables..
  cli
  jmp newgame

;------------------------------------------------------------------------------

.if USE_HIRES=1		; Use hi-res (2x3,C64)

	; Hi-res piece data for ORAC, into UDG chars 193-208. (16x8,128 bytes) copy to chrmem $3608-3688
	;  note - once copied, this mem can be reused.
	
orac_hires_pieces:				; (This is the orig 1980's hires set)

						; White Man
  .byte $00,$00,$00,$00,$00,$00,$0f,$3f,$7f,$7f,$5f,$67,$70,$7f,$7f,$3f,$0f,$00,$00,$00,$00,$00,$00,$00
  .byte $00,$00,$00,$00,$00,$00,$f8,$fe,$ff,$ff,$fd,$f3,$07,$ff,$ff,$fe,$f8,$00,$00,$00,$00,$00,$00,$00
						; White king
  .byte $00,$00,$00,$00,$0f,$3f,$7f,$7f,$5f,$67,$70,$7f,$7f,$5f,$67,$70,$7f,$3f,$0f,$00,$00,$00,$00,$00
  .byte $00,$00,$00,$00,$f8,$fe,$ff,$ff,$fd,$f3,$07,$ff,$ff,$fd,$f3,$07,$ff,$fe,$f8,$00,$00,$00,$00,$00
						; Black man
  .byte $00,$00,$00,$00,$00,$00,$0f,$38,$70,$60,$70,$78,$6f,$60,$70,$38,$0f,$00,$00,$00,$00,$00,$00,$00
  .byte $00,$00,$00,$00,$00,$00,$f8,$0e,$07,$03,$07,$0f,$fb,$03,$07,$0e,$f8,$00,$00,$00,$00,$00,$00,$00
						; Black king
  .byte $00,$00,$00,$00,$0f,$38,$70,$60,$70,$78,$6f,$60,$70,$78,$6f,$60,$70,$38,$0f,$00,$00,$00,$00,$00
  .byte $00,$00,$00,$00,$f8,$0e,$07,$03,$07,$0f,$fb,$03,$07,$0f,$fb,$03,$07,$0e,$f8,$00,$00,$00,$00,$00

.endif		; (USE_HIRES)

;-----------------------------------------------------------------------------------

.if USE_HIRES=2		; Use hi-res set 2 (2x3, VIC20) Slightly narrower

orac_hires_pieces:
			; White man (left)
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000111
  .byte %00011111
  .byte %00111111
  .byte %00111111
  .byte %00101111
  .byte %00110011
  .byte %00111100
  .byte %00111111
  .byte %00011111
  .byte %00000111
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
			; White man (right)
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %11100000
  .byte %11111000
  .byte %11111100
  .byte %11111100
  .byte %11110100
  .byte %11001100
  .byte %00111100
  .byte %11111100
  .byte %11111000
  .byte %11100000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
			; White king (left)
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000111
  .byte %00011111
  .byte %00111111
  .byte %00111111
  .byte %00101111
  .byte %00110011
  .byte %00111100
  .byte %00111111
  .byte %00101111
  .byte %00110011
  .byte %00111100
  .byte %00011111
  .byte %00000111
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
			; White king (right)  
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %11100000
  .byte %11111000
  .byte %11111100
  .byte %11111100
  .byte %11110100
  .byte %11001100
  .byte %00111100
  .byte %11111100
  .byte %11110100
  .byte %11001100
  .byte %00111100
  .byte %11111000
  .byte %11100000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
			; Black man (left)  
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00001111
  .byte %00011000
  .byte %00110000
  .byte %00100000
  .byte %00110000
  .byte %00111000
  .byte %00101111
  .byte %00100000
  .byte %00110000
  .byte %00011000
  .byte %00001111
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %11110000
  .byte %00011000
  .byte %00001100
  .byte %00000100
  .byte %00001100
  .byte %00011100
  .byte %11110100
  .byte %00000100
  .byte %00001100
  .byte %00011000
  .byte %11110000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
				; Black king
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00001111
  .byte %00011000
  .byte %00110000
  .byte %00100000
  .byte %00110000
  .byte %00111000
  .byte %00101111
  .byte %00100000
  .byte %00110000
  .byte %00111000
  .byte %00101111
  .byte %00100000
  .byte %00110000
  .byte %00011000
  .byte %00001111
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %11110000
  .byte %00011000
  .byte %00001100
  .byte %00000100
  .byte %00001100
  .byte %00011100
  .byte %11110100
  .byte %00000100
  .byte %00001100
  .byte %00011100
  .byte %11110100
  .byte %00000100
  .byte %00001100
  .byte %00011000
  .byte %11110000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  .byte %00000000
  
.endif		; (USE_HIRES)

;-----------------------------------------------------------------------------------

.if USE_HIRES=3		; Use hi-res (UDG at $3000)  (2x2)

orac_hires_pieces:
			; White  man
   .byte %00000000
   .byte %00000011
   .byte %00001111
   .byte %00011100
   .byte %00111011
   .byte %00110111
   .byte %01101110
   .byte %01101101
   .byte %01101101
   .byte %01101110
   .byte %00110111
   .byte %00111011
   .byte %00011100
   .byte %00001111
   .byte %00000011
   .byte %00000000

   .byte %00000000
   .byte %11000000
   .byte %11110000
   .byte %00111000
   .byte %11011100
   .byte %11101100
   .byte %01110110
   .byte %10110110
   .byte %10110110
   .byte %01110110
   .byte %11101100
   .byte %11011100
   .byte %00111000
   .byte %11110000
   .byte %11000000
   .byte %00000000
			; White  king
   .byte %00000000
   .byte %00000011
   .byte %00001111
   .byte %00011100
   .byte %00111011
   .byte %00110110
   .byte %01101110
   .byte %01101000
   .byte %01101000
   .byte %01101110
   .byte %00110110
   .byte %00111011
   .byte %00011100
   .byte %00001111
   .byte %00000011
   .byte %00000000
 
   .byte %00000000
   .byte %11000000
   .byte %11110000
   .byte %00111000
   .byte %11011100
   .byte %01101100
   .byte %01110110
   .byte %00010110
   .byte %00010110
   .byte %01110110
   .byte %01101100
   .byte %11011100
   .byte %00111000
   .byte %11110000
   .byte %11000000
   .byte %00000000
			; Black  man
   .byte %00000000
   .byte %00000011
   .byte %00001100
   .byte %00010000
   .byte %00100000
   .byte %00100000
   .byte %01000001
   .byte %01000010
   .byte %01000010
   .byte %01000001
   .byte %00100000
   .byte %00100000
   .byte %00010000
   .byte %00001100
   .byte %00000011
   .byte %00000000

   .byte %00000000
   .byte %11000000
   .byte %00110000
   .byte %00001000
   .byte %00000100
   .byte %00000100
   .byte %10000010
   .byte %01000010
   .byte %01000010
   .byte %10000010
   .byte %00000100
   .byte %00000100
   .byte %00001000
   .byte %00110000
   .byte %11000000
   .byte %00000000
			; Black  king
   .byte %00000000
   .byte %00000011
   .byte %00001100
   .byte %00010000
   .byte %00100000
   .byte %00100001
   .byte %01000001
   .byte %01000111
   .byte %01000111
   .byte %01000001
   .byte %00100001
   .byte %00100000
   .byte %00010000
   .byte %00001100
   .byte %00000011
   .byte %00000000

   .byte %00000000
   .byte %11000000
   .byte %00110000
   .byte %00001000
   .byte %00000100
   .byte %10000100
   .byte %10000010
   .byte %11100010
   .byte %11100010
   .byte %10000010
   .byte %10000100
   .byte %00000100
   .byte %00001000
   .byte %00110000
   .byte %11000000
   .byte %00000000
  
.endif		; (USE_HIRES)

;-----------------------------------------------------------------------------------

.if USE_HIRES		; Extra UDG characters (chr 102 on..)

hires_chars:
  .byte %10101010
  .byte %00000000
  .byte %01010101
  .byte %00000000
  .byte %10101010
  .byte %00000000
  .byte %01010101
  .byte %00000000
  
.if IS_V20		; Thin lines for Vic20

			; CHR_LINERIGHT
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
			; CHR_TOPLEFT
  .byte %11111111
  .byte %10000000
  .byte %10000000
  .byte %10000000
  .byte %10000000
  .byte %10000000
  .byte %10000000
  .byte %10000000
			; CHR_TOPRIGHT
  .byte %11111111
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001
  .byte %00000001

.else
			; CHR_LINERIGHT
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
			; CHR_TOPLEFT
  .byte %11111111
  .byte %11111111
  .byte %11000000
  .byte %11000000
  .byte %11000000
  .byte %11000000
  .byte %11000000
  .byte %11000000
			; CHR_TOPRIGHT
  .byte %11111111
  .byte %11111111
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
  .byte %00000011
.endif

.endif

;------------------------------------------------------------------------------
prgend = *		; Program end. (prglen = prgend-prgstart)
prglen = prgend-prgstart+256

; Place stacks and other tables after code mem end.

  tinterest=	PHEAP				; 
  tcolor=	PHEAP +  MAX_PLY 		; 
  tmoveptrlo=	PHEAP + (MAX_PLY * 2)		; Position this move in tmovestack
  tmoveptrhi=	PHEAP + (MAX_PLY * 3)		; 
  tcurmove=	PHEAP + (MAX_PLY * 4)		; 
  tnomoves=	PHEAP + (MAX_PLY * 5)		; 
  tmoveevallo=	PHEAP + (MAX_PLY * 6)		; 
  tmoveevalhi=	PHEAP + (MAX_PLY * 7)		; 
  ttakenpiece=	PHEAP + (MAX_PLY * 8)		; 
  tspotevallo=	PHEAP + (MAX_PLY * 9)		; 
  tspotevalhi = tspotevallo + MAX_SPOTTAB		; 
  tabbvallo = tspotevalhi + MAX_SPOTTAB

	; Scavenge lower memory for more search-stack storage

  tbestmove  = mem_tape + MAX_PLY 			; 
  tbestdir   = mem_tape + (MAX_PLY * 2)
  ttime      = mem_tape + (MAX_PLY * 3)
  tmovestack = mem_tape + (MAX_PLY * 4)

  twhite  = mem_input
  tblack  = mem_input +  MAX_PLY
  tpiece  = mem_input + (MAX_PLY * 2)
  tkings  = mem_input + (MAX_PLY * 3)

  tabbmove  = mem_page2			; (PET:TA=$1f74)
  tabbdir   = mem_page2 + MAX_TAB		; (PET:TA+24)
  tabbvalhi = mem_page2 + MAX_TAB * 2		

  tdirstack = mem_stack	
  ;tabbvallo = mem_stack + MAX_MOVESTACK

